import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

public class DrawMarketLoadJPanel extends JPanel  implements ActionListener {
	ControlBoard controlBoardFrame;
   	OrderPlayersList orderPlayersList;
   	DrawMarketLoad frame;
   	Log appLog;
	   
	int OFFSET = 50;
	int POINT_SIZE = 5;
	int x, y, previousX, previousY = OFFSET;
	int SIZE_FACTOR = 2;
    int scaleX = 1;
    int scaleY = 1;
    int x0 = 0;
    int y0 = 0;
    
   	int X_INCREASE_FACTOR = 3;
	int Y_INCREASE_FACTOR = 1;
    int x0_Chart = OFFSET;
    int y0_Chart = OFFSET;
    
	Graphics2D g2d = null;
	int nbSec = 0;
    	   
    JButton buttonPlus_X, buttonMinus_X, buttonPlus_Y, buttonMinus_Y, buttonReset, button_AddRemove_AllPlayers;
    
	ListOfPlayersGraphs listOfPlayersGraphs = null;
	
	boolean display_AllPlayers = true;
    
	private final static String FILE_LOAD_CURVE_POINTS_NAME = "Load curve points.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_LOAD_CURVE_POINTS_HEADER =
														"Time" + CSV_SEPARATOR
														+ "OMT" + CSV_SEPARATOR
														+ "Player" + CSV_SEPARATOR
														+ "BG"
														+ NEW_LINE_SEPARATOR;

    // ***************************************************
    // Constructor 
    // ***************************************************
    public DrawMarketLoadJPanel(ControlBoard controlBoardFrame, DrawMarketLoad frame, OrderPlayersList orderPlayersList, Log appLog) {
        this.orderPlayersList = orderPlayersList ;
        this.frame = frame;
        this.controlBoardFrame = controlBoardFrame;
        this.appLog = appLog;
        
        listOfPlayersGraphs = new ListOfPlayersGraphs();

        buttonPlus_X = new JButton("X +"); 
        buttonMinus_X = new JButton("X -");
        buttonPlus_Y = new JButton("Y +");
        buttonMinus_Y = new JButton("Y -");
        buttonReset = new JButton("Reset zoom");
        button_AddRemove_AllPlayers = new JButton("+/- All Players"); 

        this.add(buttonPlus_X); buttonPlus_X.addActionListener(this);
 	   	this.add(buttonMinus_X); buttonMinus_X.addActionListener(this);
	   	this.add(buttonPlus_Y);  buttonPlus_Y.addActionListener(this);
	   	this.add(buttonMinus_Y); buttonMinus_Y.addActionListener(this);
	   	this.add(buttonReset); 	buttonReset.addActionListener(this);
	   	this.add(button_AddRemove_AllPlayers); 	button_AddRemove_AllPlayers.addActionListener(this);
    }

	@Override
	public void actionPerformed(ActionEvent e) {
		if (e.getSource()==buttonPlus_X) {
			X_INCREASE_FACTOR = X_INCREASE_FACTOR*2;
			this.repaint();
			}
		
		if (e.getSource()==buttonMinus_X) {
			if (X_INCREASE_FACTOR >= 2) X_INCREASE_FACTOR = X_INCREASE_FACTOR/2;
			this.repaint();
			}
		
		if (e.getSource()==buttonPlus_Y) {
			Y_INCREASE_FACTOR = Y_INCREASE_FACTOR*2;
			this.repaint();
			}
		
		if (e.getSource()==buttonMinus_Y) {
			if (Y_INCREASE_FACTOR >= 2) Y_INCREASE_FACTOR = Y_INCREASE_FACTOR/2;
			this.repaint();
			}
		
		if (e.getSource()==buttonReset) {
			SIZE_FACTOR = 2;
			X_INCREASE_FACTOR = 3;
			Y_INCREASE_FACTOR = 1;
			scaleX = 1;
			scaleY = 1;
			this.repaint();
			}	
	
		if (e.getSource()==button_AddRemove_AllPlayers) {
			if (display_AllPlayers) display_AllPlayers = false;
			else display_AllPlayers = true;
			this.repaint();
			}
	}
	
	 
	   // ***********************************************************
	   //  drawLoad
	   // 
	   // ***********************************************************
	    void drawLoad(Graphics g) {
	      
	    	g2d = (Graphics2D) g;
	    	g2d.setPaint(Color.black);
	    	g2d.setStroke(new BasicStroke(2));
	    	boolean bScaleChange = false;
	        // Y axis

	        String sCurrentCount =""; // string to display the last point X,Y values
	       
	        // point 0
			x0 = OFFSET+POINT_SIZE/3;
			y0 = this.getHeight()-OFFSET;
			nbSec = 0;
	                
			// bigger black Font
	    	g2d.setPaint(Color.black);
			Font stringFont = new Font( "SansSerif", Font.BOLD, 13);
	    	g2d.setFont(stringFont);
	    	
    		initChart(g2d,x0,y0);
			
	        if (orderPlayersList != null) {
	    		// initialize chart
	        	if (orderPlayersList.getNbPlayers() == 0) {

	        		
	        	}
	        	else { // at least one player
	        		// calculate the new point (x,y) for the total load (all players) with current conditions
	        		
	        		nbSec = (int)(orderPlayersList.getElapsedTimeMsSinceStart()/ 1e9);
		            
		            // update the graph points for the aggregation of players
		            int displaySpeedFactor = 3;
		            x = x0+displaySpeedFactor*nbSec ;
		            y = y0-(displaySpeedFactor*(int)(orderPlayersList.getNbTotalOrderM7()));
		            
		    		// **********************************************
		            // add the new point to  the graph representing ALL Players (sum of all players = total load) and to all players graph (even if unchanged for a given player, just write again the same value with current time)
		    		// ***********************************************
		            String timeString = orderPlayersList.getLastUpdateTimeString();
		            GraphPoint graphPoint_ALL = new GraphPoint(x, y, timeString, Color.black,orderPlayersList.getNbTotalOrderM7() ); 

		            listOfPlayersGraphs.updatePlayerGraph("ALL_PLAYERS", "ALL_BG", graphPoint_ALL);
		            
		            // update all players graph for players listed in orderPlayersList
		            // (even if only one player really gets updated, 
		            // the point being to have for each x value an y value to draw a chart that shows some continuity for each player)
		    		for (OrderPlayer nextPlayer : orderPlayersList.getOrderPlayersList()) {
		    			
		                int yPlayer = y0-(displaySpeedFactor*(int)(nextPlayer.getNbofUserOrderEventsTotal()));
		                
		                GraphPoint graphPoint_Player = new GraphPoint(x, yPlayer, timeString, nextPlayer.getPlayerChartColor(),nextPlayer.getNbofUserOrderEventsTotal());
		    			listOfPlayersGraphs.updatePlayerGraph(nextPlayer.getMemberName(),nextPlayer.getBalancingGroup() , graphPoint_Player);
		    			
		            }  
		            // ********************************************************
		            // reframe the graph if it's getting too close to borders
		    		// ********************************************************
		            // resize X
		            int resizeTrials = 1;
		            int xTemp = x;

		            // adjust scale X until the new point stays within the graph boundaries
	        		while ((x0+(xTemp-x0)/scaleX)>(this.getWidth()-4*OFFSET) ) { 
	        			bScaleChange = true;
	        	        if (bScaleChange == true) {
	        	        		//displayChart (g2d, true, Color.lightGray, appLog);
	        	        		initChart(g2d,x0,y0);
	        	        }
	        			resizeTrials++;
	        			if (resizeTrials>10000) break;
	        			//appLog.logger.info("Draw point determine scaleX apply Size Factor: old xTemp = " + xTemp + " (x=" + x + ") scaleX = "+scaleX);
	    				// recalculate all points applying the new factor to get a smaller curve
	    				//recalculateGraphPointsListValues (x0, y0, SIZE_FACTOR);        			
	    				
	    				// recalculate current point with the new size factor
	    				scaleX = scaleX*SIZE_FACTOR;    				
	    				xTemp = x0+(xTemp-x0)/scaleX ;
	    	            //appLog.logger.info("Draw point apply Size Factor: newxTempx = " + xTemp + " scaleX = "+scaleX);
	        		}
	        		// resize Y
		            resizeTrials = 1;
		            int yTemp = y;
	        		while ( (y0 - (y0-yTemp)/scaleY)<OFFSET ) {
	        			bScaleChange = true;
	        	        if (bScaleChange == true) {
	        	        		//displayChart (g2d, true, Color.lightGray, appLog);
	        	        		initChart(g2d,x0,y0);
	        	        }
	        			resizeTrials++;
	        			if (resizeTrials>10000) break;
	        			//appLog.logger.info("Draw point apply Size Factor: resizeTrials= " + resizeTrials + " old yTemp = "+ yTemp + " (y=" + y + ") scaleY = "+scaleY);
	    				// recalculate all points applying the new factor to get a smaller curve
	    				//recalculateGraphPointsListValues (x0, y0, SIZE_FACTOR);        			
	    				
	    				// recalculate current point with the new size factor
	    				scaleY = scaleY*SIZE_FACTOR;    				
	    				yTemp = y0-(y0-yTemp)/scaleY;
	    	           // appLog.logger.info("Draw point apply Size Factor: new yTemp = "+ yTemp + " scaleY = "+scaleY);
	        		}
	        		// scale X and Y are determined
	           	}
	         }
	               
	    
			// ***************************************
			// redraw the axes labels with new stats
			// ***************************************
	       	// display in bold values for the last point in the upper left corner of the chart
			g2d.setFont( new Font( "SansSerif", Font.BOLD, 14) );
			
	 		// X = TIME
	  		g2d.setFont( new Font( "SansSerif", Font.BOLD, 14) );
	  		sCurrentCount = "X: " +(nbSec)/60+ " mns (" + (nbSec) + " sec)";
	  		g2d.drawString(sCurrentCount, 3*this.getWidth()/4, this.getHeight()-33);
	  		// overwriteText( g2d, sCurrentCount, Color.black, Color.lightGray, 2*this.getWidth()/4, this.getHeight()-33);
	  		
	  		g2d.setFont( new Font( "SansSerif", Font.PLAIN, 14) );
			sCurrentCount = "ScaleX=1/" + scaleX;
			g2d.drawString(sCurrentCount, 3*this.getWidth()/4, this.getHeight()-15);
			//overwriteText( g2d, sCurrentCount, Color.black, Color.lightGray, 2*this.getWidth()/4, this.getHeight()-15);
			
	       // Y = LOAD
			g2d.setFont( new Font( "SansSerif", Font.BOLD, 14) );
	   		int tempNbSec = 1;
	   		if (nbSec > 0) {
	   			tempNbSec = nbSec;
	   		}
	   		sCurrentCount =   "Y: User Load = "+orderPlayersList.getNbTotalOrderM7() + " ("+ (int)((orderPlayersList.getNbTotalOrderM7()*60*60/ tempNbSec)) + "/hour) ";
	   		g2d.drawString(sCurrentCount, OFFSET +10, 70);
	   		//overwriteText( g2d, sCurrentCount, Color.black, Color.lightGray, OFFSET +10, 70);

	   		g2d.setFont( new Font( "SansSerif", Font.PLAIN, 14) );
	   		sCurrentCount = "scaleY=1/" + scaleY;
	   		g2d.drawString(sCurrentCount,OFFSET +10, 70+20);
	   		//overwriteText( g2d, sCurrentCount, Color.black, Color.lightGray, OFFSET +10, 70+20);

	   		// *****************************
	        // draw lines between all points
	   		// *****************************
	   		
	        if (bScaleChange == true) { // need to resize the chart : draw the full chart
	     	   displayChart (g2d, false, Color.black, appLog,tempNbSec);
	        }
	        else { // only draw the last point
	     	  displayChart (g2d, false, Color.black, appLog,tempNbSec);
	        }
	    }
	    
	    // ***********************************************************
	    // draw the chart in the indicated color, 
	    // with an option to only display the last 2 points (avoid the blinking effect to moments where the chart needs to be resized)
	    // ***********************************************************
	    public void displayChart (Graphics2D g2d, boolean bDrawFullChart, Color defaultColor, Log appLog, int nbSec) {

	        int playerNb = 0;
	        g2d.setPaint(defaultColor);
	        
	        // if (appLog!= null) appLog.logger.info("Draw curve");
	        // for each player with a graph
	        for (PlayerGraph nextPlayerGraph : this.listOfPlayersGraphs.getListOfPlayersGraphs()) {
	        	playerNb ++;
	            //   g2d.setPaint(color);
	        	if (playerNb == 1) { 
	            	g2d.setStroke(new BasicStroke(3)); // total LOAD in bold
	        	}
	        	else  {
	        		 g2d.setStroke(new BasicStroke(2));
	        	}
	   	
	            //	draw lines between all points
	            int pointNb = 1;
	            GraphPoint previousPoint = null;
	            int previousPointX 	= 0;
	            int currentPointX 	= 0;
	            int previousPointY 	= 0;
	            int currentPointY 	= 0;
	            boolean bDrawLine = true;

	    		if (playerNb == 1 && (orderPlayersList.getNbPlayers() == 1 || !display_AllPlayers) ) {
	    			
	    		}
	    		else {
		        	 // draw the graph for this player
			        for (GraphPoint next : nextPlayerGraph.getGraphPointsList()) {
			        	bDrawLine = true;
			        	g2d.setPaint(next.getColor());
			        	if (pointNb == 1) {
			        		// draw a single point and not a line for the 1st point
			        		previousPointX = next.getX();
			        		previousPointY = next.getY();
			        		previousPoint = new GraphPoint();
			        	}
			    		currentPointX = next.getX();
			    		currentPointY = next.getY();
			        	
			        	// if not the last point do not draw the line when the full chart is not requested
			        //	if (bDrawFullChart==false && pointNb < nextPlayerGraph.getGraphPointsList().size()) bDrawLine = false;
			        	
		    		
			        	// force to draw each line when the chart is resized: bDrawLine set to true
			        	if (bDrawLine == true) {
				        	int x1 = x0 + X_INCREASE_FACTOR*(previousPointX-x0)/scaleX;
				        	int x2 = x0 + X_INCREASE_FACTOR*(currentPointX-x0)/scaleX;   	
				        	int y1 = y0 - Y_INCREASE_FACTOR*(y0-previousPointY)/scaleY;
				        	int y2 = y0 - Y_INCREASE_FACTOR*(y0-currentPointY)/scaleY;
				        	
			        		// label which player
				        	//overwriteText( g2d, nextPlayerGraph.getPlayer(), Color.lightGray, Color.lightGray, x2, y2);
				        	if (orderPlayersList.getNbPlayers() == 2 && playerNb == 1 ) {
				        		// do not display the ALL PLAYER chart in case there is only one real player : skip the ALL_PLAYER chart drawing
				        	}
					        	if (playerNb > 0 && playerNb < 100) {g2d.drawLine(x1, y1, x2, y2);}
				        		
				        		Font stringFont = new Font( "SansSerif", Font.BOLD, 12);
				            	g2d.setFont(stringFont);

			            		if (playerNb > 0 && playerNb < 100 && pointNb == nextPlayerGraph.getGraphPointsList().size()) {
			            			g2d.setColor(next.getColor());
				            		String labelGraphPlayer = nextPlayerGraph.getPlayer() + " : " +  next.getNbofUserOrderEventsTotal() 
	        								+  " OMT ("+ (int)((next.getNbofUserOrderEventsTotal()/ nbSec)*60*60) + " / hour)" ;
		            				//g2d.drawString(labelGraphPlayer , this.getWidth()-(int)5*OFFSET, y2);
				            		g2d.drawString(labelGraphPlayer , x2+10, y2);
	
			              	}
			            	// store the previous point characteristics to be able to delete the previous label when drawing the next point
			            	// time not stored, not involved
				    		previousPoint = new GraphPoint(x2, y2, next.getTime(),next.getColor(), next.getNbofUserOrderEventsTotal());            	
			        	}
			        	// current point becomes previous point (to draw the next line)
			    		previousPointX = currentPointX;
			    		previousPointY = currentPointY;
	
			    		pointNb++;	
			        }
		        }
	        }

	    }
		// ***************************************************
		// *** initialize the chart
		// ***************************************************    
		public void initChart(Graphics2D g2d, int x0, int y0) {
	        int x1,y1,x2,y2 = 0;
	        
		   	g2d.setPaint(Color.black);
	        // Draw X axis
	        x1 = OFFSET;
	        x2 = this.getWidth()-OFFSET-10;
	        y1 = this.getHeight()-OFFSET;
	        y2 = y1;
	        g2d.drawLine(x1, y1, x2, y2);
	        
		   	// Draw Y axis
	        x1 = OFFSET;
	        x2 = x1;
	        y1 = this.getHeight()-OFFSET;
	        y2 = OFFSET;
	        g2d.drawLine(x1, y1, x2, y2);
	        
	        // display scale
	        g2d.drawString("0", x0-13, y0+5 );
	       // g2d.drawString( Integer.toString((this.getHeight()-OFFSET)*scaleY), x0-30, 70 );
	        // g2d.drawLine(x0-5,70,x0+5,70);
	        
			x = x0;
			y = y0;
			
		}
		// ***************************************************
		// *** to resize the chart
		// ***************************************************    
	    public void IncreaseScaleY( ) {
	    	this.scaleY = this.scaleY*2;
	    }	    
	 	// ***************************************************
	 	// *** getXChart
	 	// ***************************************************
	    public int getXChart(int X) {
	    	int XChart = x0_Chart + (int)(X_INCREASE_FACTOR*(X/scaleX)) ;
	    	return XChart;
	    }
	 	// ***************************************************
	 	// *** getYChart
	 	// ***************************************************
	    public int getYChart(int Y) {
	    	
	    	int YChart = (this.getHeight() - y0_Chart) - ((Y * Y_INCREASE_FACTOR)/scaleY) ;
	    	//System.out.println("this.getHeight() = "+ this.getHeight() + " y0_Chart = "+ y0_Chart + " Y = "+ Y + " Y_INCREASE_FACTOR= " + Y_INCREASE_FACTOR  + " result =" + YChart);
	    	return YChart;
	    }
	    
	    
		// ***************************************************
		// *** WritePlayersListCSVFile
		// ***************************************************	
		public void WritePlayersListCSVFile() {
			try {
		         File f = new File(FILE_LOAD_CURVE_POINTS_NAME);
		         if(!f.exists()) {
		            f.createNewFile();
		         }
		         
				FileWriter fileWriter = new FileWriter(FILE_LOAD_CURVE_POINTS_NAME);
				
				if(f.length() == 0) fileWriter.append(FILE_LOAD_CURVE_POINTS_HEADER);
					
				int i = 0;
		        for (PlayerGraph nextPlayerGraph : this.listOfPlayersGraphs.getListOfPlayersGraphs()) {			
					for (GraphPoint next : nextPlayerGraph.getGraphPointsList()) {

						String row = 	next.getTime() + CSV_SEPARATOR
										+ next.getNbofUserOrderEventsTotal() + CSV_SEPARATOR
										+ nextPlayerGraph.getPlayer() + CSV_SEPARATOR
										+ nextPlayerGraph.getPlayerBG()
										+ NEW_LINE_SEPARATOR;
						fileWriter.append(row);
						i++;
						}	
		        }
		
				fileWriter.flush();
				fileWriter.close();	
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				JOptionPane.showMessageDialog(controlBoardFrame, "catching error: "+ e.toString()); 
			}		
		}
		// ***************************************************
		// *** WritePlayersListWrite_Historical_PlayersListCSVFileCSVFile
		// ***************************************************	
		public void Write_Historical_PlayersListCSVFile() {
			try {
			
				// write details
				String newFileName = FILE_LOAD_CURVE_POINTS_NAME+" v2.csv";
				// build the list of players for the header
				String headerPlayersList = "Time"+ CSV_SEPARATOR;
		        for (PlayerGraph nextPlayerGraph : this.listOfPlayersGraphs.getListOfPlayersGraphs()) {	
		        	headerPlayersList = headerPlayersList + nextPlayerGraph.getPlayer() + CSV_SEPARATOR;
		        }
		        headerPlayersList = headerPlayersList + NEW_LINE_SEPARATOR;
		        
		         File f = new File(newFileName);
		         if(!f.exists()) {
		            f.createNewFile();
		         }
		         
		         FileWriter fileWriter = new FileWriter(newFileName);
				
				if(f.length() == 0) fileWriter.append(headerPlayersList);
					
				
				// get the ALL PLAYERS graph list of times and pick each players value
				 String rowTime = "";
				 String row = 	"";
		        for (PlayerGraph nextPlayerGraph : this.listOfPlayersGraphs.getListOfPlayersGraphs()) {			
		        	if ("ALL_PLAYERS".contentEquals(nextPlayerGraph.getPlayer())) {
		        		// go through all time events
			        	for (GraphPoint next : nextPlayerGraph.getGraphPointsList()) {
			        		
			        		rowTime = next.getTime() ;
			        		row = rowTime + CSV_SEPARATOR;
			        		// for that time check the OMT value of each player
			        		// TO BE COMPLETED
			    	        for (PlayerGraph nextPlayerGraph2 : this.listOfPlayersGraphs.getListOfPlayersGraphs()) {			
			    	        	if (nextPlayerGraph2.getPlayer().contentEquals(nextPlayerGraph.getPlayer())) {
			    	        		// find the OMT value for that player and time
			    	        		for (GraphPoint nextPointCurrentPlayer : nextPlayerGraph.getGraphPointsList()) {
			    	        			if (nextPointCurrentPlayer.getTime().contentEquals(rowTime)) {
			    	        				long OMT_Current_Player = nextPointCurrentPlayer.getNbofUserOrderEventsTotal();
			    	        				row = row + OMT_Current_Player + CSV_SEPARATOR;
			    	        			}
			    	        		}
			    	        	}
			    	        }
			        		
							row = row + NEW_LINE_SEPARATOR;
							fileWriter.append(row);
							}	
		        		rowTime = "";
		        		
		        	break;	
		        	}
		        }
				
				fileWriter.flush();
				fileWriter.close();	
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				JOptionPane.showMessageDialog(controlBoardFrame, "catching error: "+ e.toString());
			}		
		}
		// ***************************************************
		// *** overwriteText removing the former text displayed before 
		// ***************************************************	
	    public void overwriteText(Graphics2D g2d, String sText, Color targetColor, Color tempColor, int x, int y) {
	    	
	    	g2d.setColor(tempColor);    	
			g2d.fillRect(x, y-12, x+170, y);

		   	g2d.setColor(targetColor);
			g2d.drawString(sText, x, y);
	    }

	    
	 	// ***************************************************
	 	// *** paintComponent
	 	// ***************************************************	
		protected void paintComponent(Graphics g) {
			super.paintComponent(g);
	        drawLoad(g);
	     }
}
