import java.awt.Color;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.JTable;
import javax.swing.JTextField;

public class OrderPlayersList {
	private CopyOnWriteArrayList<OrderPlayer> orderPlayersList = new CopyOnWriteArrayList();
	private SimpleDateFormat sdfPlayer = new SimpleDateFormat("yyy/MM/dd HH:mm:ss SSS") ;
	
	private int nbPlayers = 0;
	private long nbTotalOrderM7 = 0;
	private long nbTotalOrderLocal = 0;
	private long nbTotalOrderRemote = 0;
	private long elapsedTimeMsSinceLastUpdate =0; // will be used by the load chart
	private String lastUpdateTimeString ="" ;
	private long startTime =0; // will be used by the load chart
	private long elapsedTimeMsSinceStart =0; // will be used by the load chart
	
	public List<OrderPlayer> getOrderPlayersList() {
		return orderPlayersList;
	}
	public void setOrderPlayersList(CopyOnWriteArrayList<OrderPlayer> orderPlayersList) {
		this.orderPlayersList = orderPlayersList;
	}
	public void setLastUpdateTimeString(String lastUpdateTimeString) {
		this.lastUpdateTimeString = lastUpdateTimeString;
	}	
	public String getLastUpdateTimeString() {
		return lastUpdateTimeString;
	}
	public void setLastUpdateTime(String lastUpdateTimeString) {
		this.lastUpdateTimeString = lastUpdateTimeString;
	}

	public long getElapsedTimeMsSinceStart() {
		return elapsedTimeMsSinceStart;
	}
	public void setElapsedTimeMsSinceStart(long elapsedTimeMsSinceStart) {
		this.elapsedTimeMsSinceStart = elapsedTimeMsSinceStart;
	}
	public long getStartTime() {
		return startTime;
	}
	public void setStartTime(long startTime) {
		this.startTime = startTime;
	}
	public long getNbTotalOrderM7() {
		return nbTotalOrderM7;
	}
	public void setNbTotalOrderM7(long nbTotalOrderM7) {
		this.nbTotalOrderM7 = nbTotalOrderM7;
	}
	public long getNbTotalOrderLocal() {
		return nbTotalOrderLocal;
	}
	public void setNbTotalOrderLocal(long nbTotalOrderLocal) {
		this.nbTotalOrderLocal = nbTotalOrderLocal;
	}
	public long getNbTotalOrderRemote() {
		return nbTotalOrderRemote;
	}
	public void setNbTotalOrderRemote(long nbTotalOrderRemote) {
		this.nbTotalOrderRemote = nbTotalOrderRemote;
	}
	public long getElapsedTimeMsSinceLastUpdate() {
		return elapsedTimeMsSinceLastUpdate;
	}
	public void setElapsedTimeMsSinceLastUpdate(long elapsedTimeMsSinceLastUpdate) {
		this.elapsedTimeMsSinceLastUpdate = elapsedTimeMsSinceLastUpdate;
	}

	
	public int getNbPlayers() {
		return nbPlayers;
	}
	public void setNbPlayers(int nbPlayers) {
		this.nbPlayers = nbPlayers;
	}
	
	private final static String FILE_PLAYERS_STATE_NAME = "User Order Events Players State.csv";
	private final static String FILE_PLAYERS_DETAILS_NAME = "User Order Events Players Details.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_PLAYERS_LIST_HEADER = "Start Time"+CSV_SEPARATOR + 
														"Last Update"+CSV_SEPARATOR
														+ "Member ID"+CSV_SEPARATOR
														+ "Member Name"+CSV_SEPARATOR
														+ "BG"+CSV_SEPARATOR
														+ "Total"+ CSV_SEPARATOR
														+ "Local"+ CSV_SEPARATOR
														+ "Remote"+ CSV_SEPARATOR 
														+ "Total vs Market ratio" + CSV_SEPARATOR
														+ "Local vs Market ratio" + CSV_SEPARATOR
														+ "Projection 24h Total"+ CSV_SEPARATOR
														+ "Projection 24h Local" + CSV_SEPARATOR
														+ "Projection 24h Remote" 
														+ NEW_LINE_SEPARATOR;
	// ***************************************************
	// *** WritePlayersListCSVFile
	// ***************************************************	
	public void WritePlayersListCSVFile(String fileName, boolean bAppendMode) {
		try {
	         File f = new File(fileName);
	         if(!f.exists()) {
	            f.createNewFile();
	         }
	         
			FileWriter fileWriter = new FileWriter(fileName, bAppendMode);
			
			if(f.length() == 0) fileWriter.append(FILE_PLAYERS_LIST_HEADER);
				
			for (OrderPlayer next : orderPlayersList) {
				String row = 	next.getStartTime_String() + CSV_SEPARATOR
								+ next.getLastUpdate_String() + CSV_SEPARATOR
								+ next.getMemberID() + CSV_SEPARATOR
								+ next.getMemberName() + CSV_SEPARATOR 
								+ next.getBalancingGroup() + CSV_SEPARATOR
								+ next.getNbofUserOrderEventsTotal() + CSV_SEPARATOR
								+ next.getNbofUserOrderEventsLocal() + CSV_SEPARATOR
								+ next.getNbofUserOrderEventsRemote() + CSV_SEPARATOR
								+ next.getPlayer_Nb_User_Order_vs_Market_Total_Order_Ratio() + CSV_SEPARATOR
								+ next.getPlayer_Nb_User_Order_vs_Market_Local_Order_Ratio() + CSV_SEPARATOR
								+ next.getPlayer_Nb_User_Order_vs_Market_Remote_Order_Ratio() + CSV_SEPARATOR 
								+ next.getNbofUserOrderEventsTotalProjection()  + CSV_SEPARATOR
								+ next.getNbofUserOrderEventsLocalProjection()  + CSV_SEPARATOR
								+ next.getNbofUserOrderEventsRemoteProjection()
								+ NEW_LINE_SEPARATOR;
				fileWriter.append(row);
				}	
			
			fileWriter.flush();
			fileWriter.close();				
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}
	
	// ***************************************************
	// *** Add a market player to the list 
	// ***************************************************
	public int AddPlayer(long startTimeLong, String startTimeString, long lastUpdateLong, String lastUpdateString, 
			String BG, String memberID, String memberName, 
			int totalOrders, int localOrders, int remoteOrders,
			String totalRatio, String localRatio, String remoteRatio, 
			BG_Member_List bgMemberList,Log appLog) {
		
		OrderPlayer  player = new OrderPlayer();
		
		// pick up a random color for the graph
		int nbOfPlayers = this.getNbPlayers();
		
		if (nbOfPlayers == 1) player.setPlayeChartColor(Color.blue);
		else if (nbOfPlayers == 2) player.setPlayeChartColor(Color.red);
		else if (nbOfPlayers == 3) player.setPlayeChartColor(Color.black);
		else if (nbOfPlayers == 4) player.setPlayeChartColor(Color.cyan);
		else if (nbOfPlayers == 5) player.setPlayeChartColor(Color.darkGray);
		else if (nbOfPlayers == 6) player.setPlayeChartColor(Color.magenta);
		else if (nbOfPlayers == 7) player.setPlayeChartColor(Color.blue);
		else if (nbOfPlayers == 8) player.setPlayeChartColor(Color.white);
		else if (nbOfPlayers == 9) player.setPlayeChartColor(Color.BLACK);
		//else if (nbOfPlayers == 10) player.setPlayeChartColor(Color.BLACK);
		else {
			int r = 100;
			int g = 100;
			int b = 200 + (int) Math.round(Math.random() * 50);

			player.setPlayeChartColor(new Color(r, g, b));			
		}
		
		// find the member ID corresponding to the BG
		BG_Member bgMember = new BG_Member();

		player.setStartTime_Long(startTimeLong);
		player.setStartTime_String(startTimeString);
		player.setLastUpdate_Long(lastUpdateLong);
		player.setLastUpdate_String(lastUpdateString);
		this.lastUpdateTimeString = lastUpdateString;
		player.setBalancingGroup(BG);
		bgMember = bgMemberList.ReturnBGMemberInfoList(BG,appLog);
		player.setMemberID(bgMember.MemberID);
		player.setMemberName(bgMember.MemberName);		
		player.setNbofUserOrderEventsTotal(totalOrders);
		player.setNbofUserOrderEventsLocal(localOrders);
		player.setNbofUserOrderEventsRemote(remoteOrders);
		player.setPlayer_Nb_User_Order_vs_Market_Total_Order_Ratio(totalRatio);
		player.setPlayer_Nb_User_Order_vs_Market_Local_Order_Ratio(localRatio);
		player.setPlayer_Nb_User_Order_vs_Market_Remote_Order_Ratio(remoteRatio);
		
		orderPlayersList.add(player);
		this.setNbPlayers(this.getNbPlayers() + 1);
		Globals.Increment_OMT(0, 0, 0, 0, 0, 0, 1, "");
		
		return this.nbPlayers;
	}
	// ************************************************************************
	// *** Update a market player in the list: add or update if BG already listed 
	// ************************************************************************
	public boolean UpdatePlayersList(long lastUpdateLong, String lastUpdateString, String BG, boolean isXBID_Order, 
			long totalMarket, int localMarket, int remoteMarket, BG_Member_List bgMemberList,Log appLog ) {

		boolean BG_Found = false;
		float totalRatioPlayer = 0;
		float localRatioPlayer = 0;
		float remoteRatioPlayer = 0;
		long timeBetweenUpdates = 0;

		for (OrderPlayer next : this.orderPlayersList) {
			
			// A. Calculate current player new values
			if ( next.getBalancingGroup().contentEquals(BG)) {
				// BG already listed : just update values
				BG_Found = true;
				timeBetweenUpdates = lastUpdateLong - next.getLastUpdate_Long(); 	
				this.elapsedTimeMsSinceLastUpdate = timeBetweenUpdates;
				if (this.startTime == 0) this.startTime = lastUpdateLong; 
				this.elapsedTimeMsSinceStart = System.nanoTime() - this.startTime;
				//System.out.println("elapsedTimeMsSinceStart=" + elapsedTimeMsSinceStart + " = "+ System.nanoTime() + "-" + this.startTime);
				//this.elapsedTimeMsSinceStart = System.currentTimeMillis() - this.startTime;
				
				this.lastUpdateTimeString = lastUpdateString;
				next.setLastUpdate_Long(lastUpdateLong);
				next.setLastUpdate_String(lastUpdateString);
				
				next.setNbofUserOrderEventsTotal(next.getNbofUserOrderEventsTotal()+1);
				if (!isXBID_Order) next.setNbofUserOrderEventsLocal(next.getNbofUserOrderEventsLocal()+1);
				if (isXBID_Order) next.setNbofUserOrderEventsRemote(next.getNbofUserOrderEventsRemote()+1);

				// calculate activity period
				long activityTimeSec = (long)((next.getLastUpdate_Long() - next.getStartTime_Long()) / 1e9);
				String activityTime = (activityTimeSec/60)+"mn (="+ activityTimeSec+ " sec)";
				next.setActivityTime_String(activityTime);
				
				//System.out.println("=> server-timestamp usage: BG = " + BG+ " lastUpdateString = "+ lastUpdateString + " System.nanoTime()="+ System.nanoTime() + " this.startTime="+ this.startTime + " elapsedTimeMsSinceStart= "+ elapsedTimeMsSinceStart+ " activityTimeSec= "+ activityTimeSec+ " activityTime="+activityTime);
			}
		}
		// if BG could not be found in the list : add it + related stats to the Order Players list
		if (BG_Found == false) {
			// find memberID, memberName based on BG
			String memberID = "TBD";
			String memberName = "TBD";
			// local order
			if (!isXBID_Order) this.AddPlayer(lastUpdateLong, lastUpdateString,lastUpdateLong, lastUpdateString, BG, memberID, memberName, 1, 1, 0, "TBD","TBD","TBD",bgMemberList, appLog);
			// remote/XBID order
			if (isXBID_Order)  this.AddPlayer(lastUpdateLong, lastUpdateString,lastUpdateLong, lastUpdateString, BG, memberID, memberName, 1, 0, 1, "TBD","TBD","TBD",bgMemberList,appLog);
		}
		// B. Calculate all values that depend on any player activity
		for (OrderPlayer next : this.orderPlayersList) {		
			// ratios
			totalRatioPlayer = ((float)next.getNbofUserOrderEventsTotal()/(float)totalMarket)*100; 
			localRatioPlayer = ((float)next.getNbofUserOrderEventsLocal()/(float)localMarket)*100;
			remoteRatioPlayer =((float)next.getNbofUserOrderEventsRemote()/(float)remoteMarket)*100;
	
			next.setPlayer_Nb_User_Order_vs_Market_Total_Order_Ratio(String.format("%02.1f%%",totalRatioPlayer));
			next.setPlayer_Nb_User_Order_vs_Market_Local_Order_Ratio(String.format("%02.1f%%",localRatioPlayer));
			next.setPlayer_Nb_User_Order_vs_Market_Remote_Order_Ratio(String.format("%02.1f%%",remoteRatioPlayer));
		}
		refreshPlayersList(totalMarket, localMarket, remoteMarket);
		
		// write the details to a file
		//WritePlayersListCSVFile(FILE_PLAYERS_DETAILS_NAME, true);
		
		return BG_Found;
	}
	// ************************************************************************
	// *** refresh Players list by updating all stats based on current time
	// ************************************************************************
	public void refreshPlayersList(long totalMarket, int localMarket, int remoteMarket) {
		float projectionTotalPlayer = 0;
		float projectionLocalPlayer = 0;
		float projectionRemotePlayer = 0;
		
		long nbTotalOrderM7 = 0;
		long nbTotalOrderLocal = 0;
		long nbTotalOrderRemote = 0;
		long elapsedTimeMsSinceLastUpdate =0;
	
		// B. Calculate all values that depend on any player activity
		for (OrderPlayer next : this.orderPlayersList) {
			
			// update global stats
			nbTotalOrderM7 = nbTotalOrderM7 + next.getNbofUserOrderEventsTotal();
			
			// calculate silence period in seconds for all players
			long silencePeriodSec = (long)((System.nanoTime() - next.getLastUpdate_Long()) / 1e9 );
			next.setSilencePeriod_String((silencePeriodSec/60)+"mn (="+ silencePeriodSec+ " sec)");
			
			//calculate 24h projections based elapsed time since startTime
			long elapsedTimeSec = (long)((System.nanoTime() - next.getStartTime_Long()) / 1e9);
			
			if (elapsedTimeSec>0) {
				projectionTotalPlayer = (next.getNbofUserOrderEventsTotal()*3600*24)/elapsedTimeSec;
				projectionLocalPlayer = (next.getNbofUserOrderEventsLocal()*3600*24)/elapsedTimeSec;
				projectionRemotePlayer = (next.getNbofUserOrderEventsRemote()*3600*24)/elapsedTimeSec;
			
			next.setNbofUserOrderEventsTotalProjection((int)projectionTotalPlayer);
			next.setNbofUserOrderEventsLocalProjection((int)projectionLocalPlayer);
			next.setNbofUserOrderEventsRemoteProjection((int)projectionRemotePlayer);
			}
		}
		// update global stats
		this.nbTotalOrderM7 = nbTotalOrderM7;
	}
	// ************************************************************************
	// *** Update the GUI table displayed Players data 
	// ************************************************************************
	public void updateGUITable(JTable playersTable, JTextField commentTextPlayers) {
		int playerRow = 0;
		int columnNb = 0;
		long totalNbM7 = 0;
		int projectionTotalNbM7 = 0;
		// format
		NumberFormat format = NumberFormat.getNumberInstance();

		for (OrderPlayer next : orderPlayersList) {
			// only update relevant values from the left to the right
			columnNb = 0;
			
			totalNbM7  = totalNbM7 + next.getNbofUserOrderEventsTotal();
			projectionTotalNbM7 = projectionTotalNbM7 + next.getNbofUserOrderEventsTotalProjection();

			playersTable.setValueAt(next.getMemberName()		,playerRow,columnNb); columnNb++ ;
			playersTable.setValueAt(next.getMemberID()			,playerRow,columnNb); columnNb++ ;
			playersTable.setValueAt(next.getBalancingGroup()	,playerRow,columnNb); columnNb++ ;
			
			playersTable.setValueAt(String.format("%06d", next.getNbofUserOrderEventsTotal()),playerRow,columnNb); columnNb++ ;
			playersTable.setValueAt(next.getPlayer_Nb_User_Order_vs_Market_Total_Order_Ratio(),playerRow,columnNb); columnNb++ ;
			
			playersTable.setValueAt(next.getStartTime_String(),playerRow,columnNb); columnNb++ ; 
			playersTable.setValueAt(next.getLastUpdate_String(),playerRow,columnNb); columnNb++ ; 
			playersTable.setValueAt(next.getActivityTime_String()	,playerRow,columnNb); columnNb++ ; 			
			playersTable.setValueAt(next.getSilencePeriod_String() ,playerRow,columnNb); columnNb++ ; 


			playersTable.setValueAt(String.format("%06d", next.getNbofUserOrderEventsLocal()),playerRow,columnNb); columnNb++ ; 
			playersTable.setValueAt(String.format("%06d", next.getNbofUserOrderEventsRemote()),playerRow,columnNb); columnNb++ ;
			
			playersTable.setValueAt(next.getPlayer_Nb_User_Order_vs_Market_Local_Order_Ratio(),playerRow,columnNb); columnNb++ ; 
			playersTable.setValueAt(next.getPlayer_Nb_User_Order_vs_Market_Remote_Order_Ratio(),playerRow,columnNb); columnNb++ ; 
			playersTable.setValueAt(String.format("%1$" + 12 + "s",format.format(next.getNbofUserOrderEventsTotalProjection())),playerRow,columnNb); columnNb++ ; 
			playersTable.setValueAt(String.format("%1$" + 12 + "s",format.format(next.getNbofUserOrderEventsLocalProjection())),playerRow,columnNb); columnNb++ ; 
			playersTable.setValueAt(String.format("%1$" + 12 + "s",format.format(next.getNbofUserOrderEventsRemoteProjection())),playerRow,columnNb); columnNb++ ; 
			playerRow++;
		}
		/*
		 * int rowTotal = orderPlayersList.size()+2; // clear the TOTAL cells to avoid
		 * interfering with the automatic sorting
		 * playersTable.setValueAt("",rowTotal,6);
		 * playersTable.setValueAt("",rowTotal,7);
		 * playersTable.setValueAt("",rowTotal,13);
		 */ 
		 // sort on "NbofUserOrderEventsTotal" column
		  playersTable.getRowSorter().toggleSortOrder(3);
		  playersTable.getRowSorter().toggleSortOrder(3);
		  
		 /* playersTable.setValueAt("TOTAL:",rowTotal,6);
		 * playersTable.setValueAt(totalNbM7,rowTotal,7);
		 * playersTable.setValueAt(projectionTotalNbM7,rowTotal,13);
		 */
		commentTextPlayers.setText(orderPlayersList.size()+ " member(s) in action");
		// NbofUserOrderEventsTotal
	/*	int valSum = 0;
		valSum = sumColumnsValues(playersTable,playerRow, 7);
		playersTable.setValueAt(valSum,playerRow+1,7);
		// 24h Projection total: column 13
		valSum = sumColumnsValues(playersTable,playerRow, 13);
		playersTable.setValueAt(valSum,playerRow+1,13);*/
				
		WritePlayersListCSVFile(FILE_PLAYERS_STATE_NAME, false);
	}
	
	// ************************************************************************
	// *** Sum column values
	// ************************************************************************
	public int sumColumnsValues(JTable table, int sumUntilRowNb, int col) {
		int sumColumn = 0;
		for (int row = 0; row < sumUntilRowNb; row++) {
			sumColumn = sumColumn + Integer.valueOf((String) table.getValueAt(row,col));
        }
		return sumColumn;
	}
	
	// ************************************************************************
	// *** Clean the entire table
	// ************************************************************************
	public void cleanGUITable(JTable table, int tableNbOfRows, int tableNbOfColumns) {

		for (int row = 0; row < tableNbOfRows; row++) {
			for (int col = 0; col < tableNbOfColumns; col++) {
				table.setValueAt("",row,col);
	        }			
        }

	}
}
