import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;
import java.util.Random;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class OrdersList {
	//private List<Order> ordersList = new ArrayList<Order>();
	private CopyOnWriteArrayList<Order> ordersList = new CopyOnWriteArrayList<>();
	public int nbOrders = 0;
	private JTable ordersTable = null;
	private JTextField commentTextOrders = null;
	
	private final static String FILE_ORDERS_NAME = "List of managed orders.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_ORDERS_HEADER = 
														"Correlation Id"+CSV_SEPARATOR
														+"Action"+CSV_SEPARATOR											
														+"fileOrderId"+CSV_SEPARATOR  
														+"clientOrderId"+CSV_SEPARATOR
														+ "Locked" + CSV_SEPARATOR
														+ "m7 ErrRespText"+CSV_SEPARATOR
														+"m7OrderId"+CSV_SEPARATOR
														+"parentOrderId"+CSV_SEPARATOR
														+"initialOrderId"+CSV_SEPARATOR
														+ "m7RevisionNo" +CSV_SEPARATOR
														+ "nb of OMT" +CSV_SEPARATOR
														+ "Java Client Sending time" +CSV_SEPARATOR
														+ "Java Client Reception time" + CSV_SEPARATOR
														+ "Client RTT" + CSV_SEPARATOR
														+ "AMQP Server Sending Timestamp" + CSV_SEPARATOR
														+ "M7 Timestamp" +CSV_SEPARATOR
														+ "M7 Last Update Time" +CSV_SEPARATOR
														+ "State" +CSV_SEPARATOR
														+ "Area" +CSV_SEPARATOR
														+ "Area Name" +CSV_SEPARATOR
														+ "Area Long name" +CSV_SEPARATOR
														+ "contractId" +CSV_SEPARATOR
														+ "contract Name" +CSV_SEPARATOR
														+ "Exe Res" +CSV_SEPARATOR
														+ "product" +CSV_SEPARATOR
														+ "delivery Start" +CSV_SEPARATOR
														+ "delivery End" +CSV_SEPARATOR
														+ "side" + CSV_SEPARATOR
														+ "quantity" + CSV_SEPARATOR
														+ "price"  + CSV_SEPARATOR
														+ "text"
														+ NEW_LINE_SEPARATOR;

	// constructor
	public OrdersList(JTable ordersTable, JTextField commentTextOrders, int max_orders_row) {
		this.ordersTable = ordersTable;
		this.commentTextOrders = commentTextOrders;
	}
	
	// ***************************************************
	// *** addOrder
	// ***************************************************
	public void addOrder(String clientSendingTm, String correlationId, String side,String action, String m7orderId, String parentOrderId, String initialOrderId ,String fileOrderId, String clientOrderId, String revisionNo, String state, String BG, String area,String executionRestriction,
						String contractId, String contractName, String product, String deliveryStart, String deliveryEnd, String quantity, String price, String type, String text,
						DeliveryAreaList deliveryAreaList, GUIWindow guiWindowOrders) {
		Order order =  new Order();
		order.bLocked_WaitForOrderExeReport = true; // lock the order until the related Order Execution Report is received
		order.correlationId = correlationId;
		order.action = action;
		order.side = side;
		order.m7OrderId = m7orderId;
		order.parentOrderId = parentOrderId;
		order.initialOrderId = initialOrderId;
		order.fileOrderId = fileOrderId;
		order.clientOrderId = clientOrderId;
		order.m7RevisionNo = revisionNo;
		order.state = state;
		order.deliveryAreaId = area;
		order.executionRestriction = executionRestriction;
		order.BalancingGroup = BG;
		order.contractId = contractId;
		order.contractName = contractName;
		order.product = product ;
		order.deliveryStart = deliveryStart;
		order.deliveryEnd = deliveryEnd;
		order.quantity = quantity;
		order.price = price;
		order.type = type;
		order.clientSendingTm = clientSendingTm; // for internal orders: stores the actual sending time ; for external orders : stores the M7 order timestamp (M7 creation time)
		order.clientSendingTm_Long = System.nanoTime();
		order.OMTNb = 1;
		order.text = text;
		
		// get order delivery area details
		DeliveryArea orderAreaInfo = new DeliveryArea(); 
		if (deliveryAreaList != null) {
			orderAreaInfo = deliveryAreaList.GetAreaDetailsFromId(area);
			order.deliveryAreaName = orderAreaInfo.dlvryAreaName;
			order.deliveryAreaLongName = orderAreaInfo.dlvryAreaLongName;			
		}
		else {
			order.deliveryAreaName = "UNKNOWN null list";
			order.deliveryAreaLongName = "UNKNOWN null list";			
		}

		this.ordersList.add(order);
		this.nbOrders ++;
		//this.updateGUITable(ordersTable, commentTextOrders,MAX_ORDER_ROWS);
		
		//String msg ="\tADDED: order clientOrderId =" + clientOrderId;
		//guiWindowOrders.AddText(msg);
	}
	
	// ***************************************************
	// *** Lock an order to avoid any order mgmt action is attempted before having received the related Order Execution Report
	// ***************************************************	
	public boolean lock_Order(String orderId, String correlationId) {
		boolean bDone = false;
		for (Order next : this.ordersList) {
			if (orderId.contentEquals(next.m7OrderId) && correlationId.contentEquals(correlationId)) next.bLocked_WaitForOrderExeReport = true;
			bDone = true;
		}	
		return bDone;
	}
	
	// ***************************************************
	// *** UnLock an order 
	// ***************************************************	
	public boolean unlock_Order(String correlationId) {
		boolean bDone = true;
		for (Order next : this.ordersList) {
			if (correlationId.contentEquals(correlationId) && next.bLocked_WaitForOrderExeReport == true) next.bLocked_WaitForOrderExeReport = false;
			bDone = true;
		}	
		return bDone;
	}
	
	// ***************************************************
	// *** update Order with a specific order Id, correlation Id and Revision No
	// *** if the correlation Id matches but there is not yet any order Id, it's because it is an order creation: update the order
	// ***************************************************
	public boolean updateOrder_CorrelationId_OrderId(String correlationId, String clientOrderId, String messageServerTimestamp, String timestmp, String m7lastUpdateTm, 
			String clientReceptionTm, String revisionNo, String orderId, String action, String state, String price, String quantity, GUIWindow guiWindowOrders) throws Exception {
		boolean orderFound = true;
		String msg = "";
		String EMPTY = "";
		
		boolean debug = false;
		
	if (debug) Globals.guiWindowDebug.AddText("Step 1: Update order table for correlation Id = " + correlationId + " order Id = " + orderId + " revision No =" + revisionNo + " action =" + action + " state=" + state);
	
	int orderNb = 0;
	for (Order next : this.ordersList) {
		boolean bOrderIdFound = false;
		if (debug) Globals.guiWindowDebug.AddText("Going through orderList : order #" +  orderNb + " Order id = " + next.m7OrderId + " revisionNo = " + next.m7RevisionNo + " correlationId="+correlationId + " client Order Id="+clientOrderId);
		//msg ="order " +  orderNb + " next.clientOrderId='" + next.clientOrderId + "'" + " Order Exe Report clientOrderId='"+ clientOrderId + "'";
		//guiWindowOrders.AddText(msg);
		
		if (next.correlationId.contentEquals(correlationId) // if the correlation ID matches the current order record in the orders list
				&& (EMPTY.contentEquals(next.m7OrderId) || (orderId.contentEquals(next.m7OrderId))) // if the order Id matches (except when still waiting for a new order Id
				&& 	( "".contentEquals(next.m7RevisionNo) 
						|| ( !"".contentEquals(next.m7RevisionNo) && (Integer.parseInt(revisionNo) > Integer.parseInt(next.m7RevisionNo)) 
								) 
					) // apply order updates only in the new revisionNo is higher than the previous one (if any) 
		) {
						
			orderFound = true;
			// updates to be done whatever is the action
			next.m7RevisionNo = revisionNo;
			next.action = action;
			next.state = state;
			next.messageServerTimestamp = messageServerTimestamp; 
			next.m7timestmp = timestmp; 
			next.m7lastUpdateTm = m7lastUpdateTm;
			next.clientReceptionTm = clientReceptionTm;
			next.clientReceptionTm_Long = System.nanoTime();
			next.clientRTT = String.format("%.3f",(next.clientReceptionTm_Long - next.clientSendingTm_Long) / 1e9) + " sec";
			next.price = price;
			next.quantity = quantity;
			next.bLocked_WaitForOrderExeReport = false;
			
			// if M7 confirms the order creation: update the revisionNo and updateTime
			if (action.contentEquals("UADD")) {
				next.m7OrderId = orderId;
			}
			// if any other user action than UADD, increment the nb of OMT (the initial UADD has already been counted)
			if ( action.contentEquals("UHIB") || action.contentEquals("UDEL") || action.contentEquals("UMOD")|| action.contentEquals("UREJ")) {
				next.OMTNb ++;
			}				
			break;
		}
		orderNb ++; // was not the order to update : go to the next order
	}
	
	if (orderFound == false) {
		if (debug) Globals.guiWindowDebug.AddText("Step 2: Update order table for correlation Id = " + correlationId + " order NOT found: adding the order");
	}
	else {
		if (debug) Globals.guiWindowDebug.AddText("Step 2: Update order table for correlation Id = " + correlationId + " order FOUND");
	}
		//displayOrdersList(guiWindowOrders);
		//this.updateGUITable(ordersTable, commentTextOrders,MAX_ORDER_ROWS);
		return orderFound;
	}
	// ***************************************************
	// *** WriteOrdersListCSVFile
	// ***************************************************	
	public void WriteOrdersListCSVFile() {
		try {
	         File f = new File(FILE_ORDERS_NAME);
	         if(!f.exists()) {
	            f.createNewFile();
	         }
	         
			FileWriter fileWriter = new FileWriter(FILE_ORDERS_NAME);
			
			if(f.length() == 0) fileWriter.append(FILE_ORDERS_HEADER);
	
			for (Order next : ordersList) {
				String row = 	next.correlationId + CSV_SEPARATOR
								+ next.action + CSV_SEPARATOR
								+ next.fileOrderId + CSV_SEPARATOR
								+ next.clientOrderId + CSV_SEPARATOR
								+ next.bLocked_WaitForOrderExeReport
								+ next.m7ErrRespText + CSV_SEPARATOR
								+ next.m7OrderId + CSV_SEPARATOR
								+ next.parentOrderId + CSV_SEPARATOR
								+ next.initialOrderId + CSV_SEPARATOR
								+ next.m7RevisionNo + CSV_SEPARATOR
								+ next.OMTNb + CSV_SEPARATOR
 								+ next.clientSendingTm + CSV_SEPARATOR
								+ next.clientReceptionTm + CSV_SEPARATOR
								+ next.clientRTT + CSV_SEPARATOR
 								+ next.messageServerTimestamp + CSV_SEPARATOR
								+ next.m7timestmp + CSV_SEPARATOR
								+ next.m7lastUpdateTm + CSV_SEPARATOR
								+ next.state + CSV_SEPARATOR
								+ next.deliveryAreaId + CSV_SEPARATOR
								+ next.deliveryAreaName + CSV_SEPARATOR
								+ next.deliveryAreaLongName + CSV_SEPARATOR
								+ next.contractId + CSV_SEPARATOR
								+ next.contractName + CSV_SEPARATOR
								+ next.executionRestriction + CSV_SEPARATOR
								+ next.product + CSV_SEPARATOR
								+ next.deliveryStart + CSV_SEPARATOR
								+ next.deliveryEnd + CSV_SEPARATOR
								+ next.side + CSV_SEPARATOR
								+ next.quantity + CSV_SEPARATOR
								+ next.price + CSV_SEPARATOR
								+ next.text
								+ NEW_LINE_SEPARATOR;

				fileWriter.append(row);
				}	
			
			fileWriter.flush();
			fileWriter.close();				
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}
	
	// ***************************************************
	// *** Display list of orders in Orders GUI
	// ***************************************************
	public void displayOrdersList(GUIWindow guiWindowOrders) {
		String msg = "\n*** display orders list ***"; 
		guiWindowOrders.AddText(msg);
		
		for (Order next : ordersList) {
			msg = "\t clientOrderId  = "+ next.clientOrderId + " M7 order Id = " + next.m7OrderId + " action = " + next.action + " contract=" + next.contractName; 
			guiWindowOrders.AddText(msg);
		}
	}
	
	// ************************************************************************
	// *** Update the GUI table displaying Order data 
	// ************************************************************************
	public void updateGUITable(GUI_Panel ownOrders_GUI_Panel) throws Exception {
		
		int orderRow = 0;
		int columnNb = 0;
		boolean alreadySortedOnce = false;
		int nbSeconds = 0;
		int nbOMT = 0;
		
		JTable ordersTable = ownOrders_GUI_Panel.table;
		JTextField commentTextOrders = ownOrders_GUI_Panel.commentText;
		DefaultTableModel dataModel = ownOrders_GUI_Panel.dataModel;
		
		// if short L1 is known then force the calculation of the nb of OMT during the past L1 seconds
		/*if (!throttlingShortL1Period.contentEquals(""))  {
			nbSeconds = Integer.valueOf(throttlingShortL1Period);
			nbOMT  = this.countUserOMTduringPeriod(nbSeconds);
			
			commentTextOrders.setText(this.nbOrders+ " order(s)" + " \t nb OMT during last " + nbSeconds + " seconds = "+ nbOMT);
		}
		else {
			commentTextOrders.setText(this.nbOrders+ " order(s)");			
		}*/
		
		commentTextOrders.setText(this.nbOrders+ " order(s)");

		for (Order next : this.ordersList) {
		
				columnNb = 0;				
				// String[] columnsOrders = "#","Correlation Id", "Client Order Id", "m7 order Id", "Ack Resp", "Error Resp",
				// "RevisionNo", "price", "qty",", "Contract Name", "Area Id",
				// "client sending","M7 AMQP sending","client reception tm","M7 Timestamp","M7 Last update"};"#","Client Order Id", "m7 order Id", "action", "Contract Name", "Area Id"};
				ordersTable.setValueAt(String.format("%04d",orderRow+1),orderRow,columnNb); columnNb++ ;
			
				ordersTable.setValueAt(next.correlationId,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.clientOrderId,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.bLocked_WaitForOrderExeReport,orderRow,columnNb); columnNb++ ;
				
				ordersTable.setValueAt(next.m7OrderId,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.parentOrderId,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.initialOrderId,orderRow,columnNb); columnNb++ ;
				String ackRespText = "";
				if (next.m7AckResp.contentEquals("1")) ackRespText = "Ack"; 
				ordersTable.setValueAt(ackRespText,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.m7ErrRespText,orderRow,columnNb); columnNb++ ;
				
				ordersTable.setValueAt(next.action,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.m7RevisionNo,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.side,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.price,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.quantity,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.text,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.OMTNb,orderRow,columnNb); columnNb++ ;

				ordersTable.setValueAt(next.contractName,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.deliveryAreaId,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.deliveryAreaName,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.deliveryAreaLongName,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.clientSendingTm,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.clientRTT,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.messageServerTimestamp,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.clientReceptionTm,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.m7timestmp,orderRow,columnNb); columnNb++ ;
				ordersTable.setValueAt(next.m7lastUpdateTm,orderRow,columnNb); columnNb++ ;

				orderRow++;	
				
				// extend the table when reaching the end 
				if  (orderRow == ordersTable.getRowCount()-1) dataModel.addRow(new Object[] { orderRow }); 
		}
		 // only sort once so that the user can change the sorting 
		 if (alreadySortedOnce == false) {
			//ordersTable.getRowSorter().toggleSortOrder(0);
			//ordersTable.getRowSorter().toggleSortOrder(0);
			alreadySortedOnce = true;
		 }
	}
	// ************************************************************************
	// *** calculate the rolling count of OMT during a period 
	// ************************************************************************
	public int countUserOMTduringPeriod(int periodNbOfSeconds) {
		int marginInSeconds = 0; // so far the bucket M7 approach is not yet implemented to really mimic the throttler count
		int nbUserOMT = 0;
		for (Order next : this.ordersList) {
			// for orders for which an Order Exe Report has been received
			//if (!next.m7OrderId.contentEquals("")) {
				
				// calculate the elapsed time since the event
				
				long elapsedSec = (long)((System.nanoTime() - next.clientSendingTm_Long) / 1e9 );
				
				if (elapsedSec < periodNbOfSeconds + marginInSeconds) {// optional margin to simplify the calculation (no bucket implementation)
					nbUserOMT = nbUserOMT + next.OMTNb;
				}
			//}
		}
		
	//commentTextOrders.setText(this.nbOrders+ " order(s)" + "\t Nb OMT during last 10 seconds = " + nbUserOMT);
	return nbUserOMT; 
	}
	
	
	// ************************************************************************
	// *** calculate the nb of Error Resp (order rejections) during a period 
	// ************************************************************************
	public int countErrRespduringPeriod(int periodNbOfSeconds) {
		int marginInSeconds = 0; // so far the bucket M7 approach is not yet implemented to really mimic the throttler count
		int nbErrRespOW = 0;
		for (Order next : this.ordersList) {
			// for orders for which an Order Exe Report has been received
			if (!next.m7ErrRespText.contentEquals("")) {
				
				// calculate the elapsed time since the event
				long elapsedSec = (long)((System.nanoTime() - next.clientSendingTm_Long) / 1e9 );
				
				if (elapsedSec < periodNbOfSeconds + marginInSeconds) { // optional margin to simplify the calculation (no bucket implementation)
					nbErrRespOW++;
				}
			}
		}
		
	return nbErrRespOW; 
	}
	// ************************************************************************
	// *** find a order with its correlationId and update the errResponse text  
	// ************************************************************************
	public boolean updateOrderErrorResponse_CorrelationId(String correlationId, String errorResponseText) {
		boolean bOrderFound = false;
		for (Order next : this.ordersList) {
			if (next.correlationId.contentEquals(correlationId)) {
				bOrderFound = true;
				next.m7ErrRespText = errorResponseText;
			}
		}
	return bOrderFound;
	}
	// ************************************************************************
	// *** find a order with its correlationId and update the Ack Resp
	// ************************************************************************
	public boolean updateOrder_AckResp_CorrelationId(String correlationId) {
		boolean bOrderFound = false;
		for (Order next : this.ordersList) {
			if (next.correlationId.contentEquals(correlationId)) {
				bOrderFound = true;
				next.m7AckResp = "1";
			}
		}
	return bOrderFound;
	}
	
	// ************************************************************************
	// *** find a order with its correlationId and update the Ack Resp
	// ************************************************************************
	public Order findAnOrderToManage() {
		Order order = new Order();
		String correlationId = "NOT_FOUND";
		 
		//int i = 0;
		
		if (this.ordersList.size()>0) {
			// random search for an order which is not in a final status
		/*	Random rdOrderPosition = new Random();
			int orderPosition = rdOrderPosition.nextInt(this.ordersList.size()-1) ;
			System.out.println("=> findAnOrderToManage orderPosition="+ orderPosition);
			Globals.guiWindowDebug.AddText("=> findAnOrderToManage orderPosition="+ orderPosition);
			*/
			
			Random randomOrderNo = new Random();
			int iTargetOrderNo = randomOrderNo.nextInt(this.ordersList.size());
			int iOrderNo = 0;
			
			for (Order next : this.ordersList) {
				//Globals.guiWindowDebug.AddText(" order #" + i + " correlation id = " + next.correlationId + " action = " + next.action);
				// check the order Id is not empty, that would mean that the order Exe Report has not been received yet :
				// that would mean that attributes generated by M7 which are mandatory to perform a modification are not yet known (order Id, revision no)
				// Note: if they are known but not yet updated then the request will be rejected (response = Error Response)
				if (iTargetOrderNo == iOrderNo
						&& !next.m7OrderId.contentEquals("") && !next.action.contentEquals("FEXE") && !next.action.contains("DEL")
						&& !next.bLocked_WaitForOrderExeReport)  {
					correlationId = next.correlationId;
					//Globals.guiWindowDebug.AddText("order found: correlation Id = "+correlationId);
					order.state = next.state; 
					order.m7OrderId = next.m7OrderId;
					order.price = next.price;
					order.quantity = next.quantity;
					order.m7RevisionNo = next.m7RevisionNo;
					order.type = next.type;
					order.executionRestriction = next.executionRestriction;
					order.action = next.action;
					order.clientOrderId = next.clientOrderId;
					order.parentOrderId = next.parentOrderId;
					order.initialOrderId = next.initialOrderId;

					break;
				}
				iOrderNo++;
			}			
		}
		else {
			//correlationId = "NOT_FOUND";
			Globals.guiWindowDebug.AddText("order not found: correlation Id = "+correlationId);
		}

		order.correlationId = correlationId;
		return order;
	
	}
}
