import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.util.Date;
import java.util.concurrent.TimeUnit;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class ParseXMLMessageRunnable implements Runnable {
	private String messageName = "";
	private AMQPMessage message = null;
	private JTable ordersTable = null;
	private JTextField commentTextOrders = null;
	private int MAX_ORDER_ROWS = 10000;
	private OrdersList ordersList = null; 
	public int nbOMTinMessage = 0;
	private GUIWindow guiWindow;
	private Log appLog = null;
	private OrderPlayersList orderPlayersList = null;
	private BG_Member_List bgMemberList = null;
	private JTable playersTable = null;
	private JTextField commentTextPlayers = null;
	private ControlBoard controlBoardFrame;
	
	long UserOrderEventsNbM7  = 0;
	int UserOrderEventsNbRemote = 0;
	int UserOrderEventsNbLocal = 0;
	
	String EPEX_ORDER_GENERATOR_BG;
	int UserOrderEventsNb_M7_EPEXOrderGenerator = 0;
	int UserOrderEventsNb_Remote_EPEXOrderGenerator = 0;
	int UserOrderEventsNb_Local_EPEXOrderGenerator = 0;
	
	private SimpleDateFormat sdfStatsWithDash = new SimpleDateFormat("yyy-MM-dd HH:mm:ss.SSS") ;
	
	Globals globals = null;
	
	// ******************************
	// constructor
	// ******************************
	public ParseXMLMessageRunnable (ControlBoard controlBoardFrame, String messageName, AMQPMessage message, 
			BG_Member_List bgMemberList, OrderPlayersList orderPlayersList, OrdersList ordersList, String EPEX_ORDER_GENERATOR_BG, 
				JTable playersTable, JTextField commentTextPlayers, GUIWindow guiWindow,Log appLog) {
		
		this.messageName 	= messageName;
		this.message 		= message;	
		this.bgMemberList = bgMemberList;
		this.ordersList 	= ordersList;
		this.orderPlayersList = orderPlayersList;
		this.EPEX_ORDER_GENERATOR_BG = EPEX_ORDER_GENERATOR_BG;
		this.playersTable = playersTable ;
		this.commentTextPlayers = commentTextPlayers ;
		this.guiWindow = guiWindow;
		this.appLog = appLog;
		this.controlBoardFrame = controlBoardFrame;
		
		ordersList = new OrdersList(ordersTable, commentTextOrders,MAX_ORDER_ROWS);
		
		if (messageName.contentEquals("FullOrdrCaptureResp")) {
			String msg = "Thread constructor correlation ID = "+ message.correlationId;
			System.out.println(msg);
			guiWindow.AddText(msg);
			
			// nbOMTinMessage = ;
		}
			}
	
	// ******************************
	// process messages
	// - order Events for FullOrdrCaptureResp
	// ******************************
	@Override
	public void run() {
		if (messageName.contentEquals("FullOrdrCaptureResp")) {
			String msg = "* Started: Thread correlation ID = "+ message.correlationId;
			System.out.println(msg);
			guiWindow.AddText(msg);	
			
			int nbOMTInMessage = 0;
			globals = new Globals(controlBoardFrame);
			long message_processing_start_time = System.nanoTime();
			
			// parse the XML message to count local and XBID/remote orders
			DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
			DocumentBuilder builder;

			try {
				builder = docFactory.newDocumentBuilder();

				// Create a Document from the XML payload
				StringBuilder xmlStringBuilder = new StringBuilder();
				xmlStringBuilder.append(message.xmlMessagePayload);
				
				// extract the server-timestamp
				// #contentHeader<basic>(content-type=x-m7/response; version=6.0, content-encoding=gzip, headers={server-timestamp=1640542630753}, ... 
				String s= message.amqpProperties;
				String prefix = "server-timestamp=";
				s = s.substring(s.indexOf(prefix) + prefix.length());
				String[] result = s.split("}");
				long messageServerTimestamp_Long = Long.parseLong(result[0]);
				String messageServerTimestamp_String = sdfStatsWithDash.format(new Date(messageServerTimestamp_Long));
				
				ByteArrayInputStream input;
				input = new ByteArrayInputStream(xmlStringBuilder.toString().getBytes("UTF-8"));

				Document document = null;
				document = builder.parse(input);
				document.getDocumentElement().normalize();

					NodeList nodeOrderList = document.getElementsByTagName("Ordr");
					
					int nbOrderNodesInMessage = nodeOrderList.getLength();
					// totalOMTcount_approximation_DayD = nbOrderNodesInMessage;

					// if there are Order nodes
					if (nbOrderNodesInMessage > 0) {

						// for each Order node
						for (int orderItr = 0; orderItr < nodeOrderList.getLength(); orderItr++)   
						{  
							msg = null;
							Node node = nodeOrderList.item(orderItr);  
							String action = "";
							boolean isXBID_Order = false;
							String orderId = null;
							String acctId = "";
							String parentOrdrId = null;
							String remoteOrdrId = null;
							String revisionNo = null;
							String clOrdrId= null;
							String timestmp = null;
							String lastUpdateTm =null;
							String initialOrdrId= null;
							String clearingAcctType = null;
							String state= null;
							String contractId = null;
							String side = null;
							String px = null;
							String qty = null;
							String initialQty = null;
							String ordrExeRestriction = null;
							String txt = null;
							String dlvryAreaId = null;
							String preArranged = null;
							String type = null;
							String usrCode = null;
							String validityDate = null;
							String validityRes = null;
							String lastUpdateUsrInfo = null;
							String aot = null;
							
							
							if (node.getNodeType() == Node.ELEMENT_NODE)   
							{  
								Element eElement = (Element) node;  
								NamedNodeMap nodeMap = eElement.getAttributes();
								// go through each attribute of the order node and extract values from relevant attributes for current order node
								for (int i = 0; i < nodeMap.getLength(); i++)   
								{  
									Node node2 = nodeMap.item(i);
									String nodeName =node2.getNodeName();
									String nodeValue =node2.getNodeValue();

									if (nodeName.contentEquals("ordrId") ) {			
										orderId = nodeValue;
									}
									if (nodeName.contentEquals("initialOrdrId") ) {			
										initialOrdrId = nodeValue;
									}	
									if (nodeName.contentEquals("parentOrdrId") ) {			
										parentOrdrId = nodeValue;
									}
									if (nodeName.contentEquals("clearingAcctType") ) {			
										clearingAcctType = nodeValue;
									}
									if (nodeName.contentEquals("clearingAcctType") ) {			
										clearingAcctType = nodeValue;
									}	
									
									if (nodeName.contentEquals("acctId") ) {
										// balancing group
										acctId = nodeValue;
									}								
									if (nodeName.contentEquals("remoteOrdrId") ) {			
										isXBID_Order = true;
										remoteOrdrId = nodeValue;
									}
									if (nodeName.contentEquals("action") ) {
										action = nodeValue;									
									}
									if (nodeName.contentEquals("revisionNo") ) {
										revisionNo = nodeValue;									
									}
									if (nodeName.contentEquals("clOrdrId") ) {
										clOrdrId = nodeValue;									
									} 
									if (nodeName.contentEquals("timestmp") ) {
										timestmp = nodeValue;									
									}
									if (nodeName.contentEquals("lastUpdateTm") ) {
										lastUpdateTm = nodeValue;									
									} 
									if (nodeName.contentEquals("contractId") ) {
										contractId = nodeValue;									
									} 
									if (nodeName.contentEquals("side") ) {
										side = nodeValue;									
									}
									if (nodeName.contentEquals("px") ) {
										px = nodeValue;									
									}
									if (nodeName.contentEquals("qty") ) {
										qty = nodeValue;									
									}
									if (nodeName.contentEquals("initialQty") ) {
										initialQty = nodeValue;									
									}
									if (nodeName.contentEquals("ordrExeRestriction") ) {
										ordrExeRestriction = nodeValue;									
									}
									if (nodeName.contentEquals("txt") ) {
										txt = nodeValue;									
									}
									if (nodeName.contentEquals("dlvryAreaIdString") ) {
										dlvryAreaId = nodeValue;									
									}
									if (nodeName.contentEquals("preArranged") ) {
										preArranged = nodeValue;									
									}
									if (nodeName.contentEquals("type") ) {
										type = nodeValue;									
									}
									if (nodeName.contentEquals("state") ) {
										state = nodeValue;									
									}
									if (nodeName.contentEquals("usrCode") ) {
										usrCode = nodeValue;									
									}
									if (nodeName.contentEquals("validityDate") ) {
										validityDate = nodeValue;									
									}
									if (nodeName.contentEquals("validityRes") ) {
										validityRes = nodeValue;									
									}
									if (nodeName.contentEquals("lastUpdateUsrInfo") ) {
										lastUpdateUsrInfo = nodeValue;									
									}
									if (nodeName.contentEquals("aot") ) {
										aot = nodeValue;									
									}
								} // for each attribute
							
							// increase the nb of Order event triggered by the user
							// only count UADD without a parent Id, to count the order start cycle
							// to avoid counting D+A events (priority change) twice (with the A having a parent Id AND a D)
							boolean bOrderEventToBeCountedAsOMT = orderEventToBeCountedAsOMT(action, parentOrdrId);
							
							if (bOrderEventToBeCountedAsOMT) {
									// count "OMT" : events triggered by the user (not the exact formula but really close)
									msg = "event to be counted : UserOrderEventsNbM7 = "+ UserOrderEventsNbM7 + ", BG = "+ acctId;
									appLog.logger.info(msg);
		
									// update the Throttling status log or the chart
									// since this event does not have any throttling data just copy the last known data (from the last Throttling status response)
									// and add specific data for the current msg

									nbOMTInMessage++;
									System.out.println(" OMT ++ = "+ nbOMTInMessage);
									UserOrderEventsNbM7 ++;
									Globals.M7_OMT ++;
									if (isXBID_Order) {
										UserOrderEventsNbRemote++;
										Globals.remote_OMT ++;
									}
									if (!isXBID_Order) {
										UserOrderEventsNbLocal++;
										Globals.local_OMT++;
									}
									Globals.guiWindowDebug.AddText("["+Thread.currentThread().getName() + "] " + message.correlationId + " : " 
															+ " M7_OMT="+   Globals.M7_OMT 
															+ " local_OMT="	+   Globals.local_OMT 
															+ " remote_OMT="+   Globals.remote_OMT);
																
									// count the contribution of the EPEX Order Generator
									if (acctId.contentEquals(EPEX_ORDER_GENERATOR_BG)) {								
										UserOrderEventsNb_M7_EPEXOrderGenerator++;
										if (isXBID_Order) UserOrderEventsNb_Remote_EPEXOrderGenerator ++;
										if (!isXBID_Order) UserOrderEventsNb_Local_EPEXOrderGenerator ++;
									}
									
									// update the Order Market Player stats list and redraw the chart
									boolean resultPlayersUpdate = false;
		
									// dates/times preparation
									Instant lastUpdateTm_Instant = Instant.parse(lastUpdateTm);
								    ZonedDateTime localLastUpdateTm = lastUpdateTm_Instant.atZone(ZoneId.systemDefault());
								    System.out.println("lastUpdateTm= " + lastUpdateTm + " CET_CEST_Time = " + localLastUpdateTm );
								    // isolate the string until the time zone correction "+"
								    String sLocalCorrectedDateTime = localLastUpdateTm.toString().substring(0,localLastUpdateTm.toString().indexOf("+") );  
		
								    // transform the last update timestamp into a long (for later time diff calculation)
									//Long lastUpdateTm_Long = sdfUTCWithDash.parse(lastUpdateTm).getTime(); // milliseconds conversion
								    Long lastUpdateTm_Long = sdfStatsWithDash.parse(messageServerTimestamp_String).getTime(); // milliseconds conversion
									Long LastUpdateTm_NanoSec_Long = TimeUnit.MILLISECONDS.toNanos(lastUpdateTm_Long);
									int tempsPasseSec = (int)((System.currentTimeMillis() - lastUpdateTm_Long) / 1e6); 
								//	System.out.println("tempsPasseSec = " + tempsPasseSec + " = "+ System.currentTimeMillis() + " - "+ lastUpdateTm_Long);
									
								  //  System.out.println("lastUpdateTm= " + lastUpdateTm + "CET_CEST_Time = " + localLastUpdateTm + " lastUpdateTm_Long="+lastUpdateTm_Long+ " nanoTime()="+System.nanoTime() + " currentMS = "+ System.currentTimeMillis());
		
									
									resultPlayersUpdate = orderPlayersList.UpdatePlayersList(System.nanoTime(),
																							sLocalCorrectedDateTime,
																							acctId, 
																							isXBID_Order, 
																							UserOrderEventsNbM7, UserOrderEventsNbLocal, UserOrderEventsNbRemote,
																							bgMemberList ,appLog);
									// **************
									if (nbOMTInMessage %1000 == 100) orderPlayersList.updateGUITable(playersTable,commentTextPlayers);
									if (nbOMTInMessage %5000 == 0) {
										orderPlayersList.updateGUITable(playersTable,commentTextPlayers);
										guiWindow.AddText("Correlation Id : "+ this.message.correlationId + " OMT =" + nbOMTInMessage);
									}
									if (nbOMTInMessage %5000 == 0) {
										System.out.println("\t ---> Writing GLOBALS file: Correlation Id : "+ this.message.correlationId + "  : M7 OMT =" + Globals.M7_OMT +  " Remote ="  + Globals.remote_OMT + " Local=" + Globals.local_OMT);
										Globals.Write_Globals_CSVFile();
									}
									
									// log count info
									msg = "USER ORDER EVENT ordrId ='"+orderId + "' action ='" + action + "' parentOrdrId = '" + parentOrdrId+  "' BG/acctId = '" + acctId + "' remote order Id (optional)='" +  remoteOrdrId+"'" + "UserOrderEventsNbM7 = "+ UserOrderEventsNbM7;
									//guiWindow.AddText(msg);
									//appLog.logger.fine(msg);
								} // if action must be counted
							
							// refresh the orders list file
							ordersList.WriteOrdersListCSVFile();

							}  // if (node.getNodeType() == Node.ELEMENT_NODE)						
						} // for each Order Exe Report Order record/node
						
					} // if there are Order nodes
			 msg = "*** end of Order Exe Report parsing: took "+ ((System.nanoTime() - message_processing_start_time) / 1e6 )+ " ms" ;
			appLog.logger.info(msg);
			guiWindow.AddText(msg);
		
			} catch (ParserConfigurationException | SAXException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				guiWindow.AddText(e.toString());
			} catch (ParseException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}				
			orderPlayersList.updateGUITable(playersTable,commentTextPlayers);
			this.nbOMTinMessage =  nbOMTInMessage;
		
		}
		Globals.Write_Globals_CSVFile();

	}

	// **************************************
	// orderEventToBeCountedAsOMT
	// **************************************	
	public boolean orderEventToBeCountedAsOMT(String action, String parentOrdrId) {
		if ((action.contentEquals("UADD")  && parentOrdrId == null) || action.contentEquals("UHIB") || action.contentEquals("UDEL") || action.contentEquals("UMOD")|| action.contentEquals("UREJ")) {
				return true;
		}
		else return false;
	}
}
