import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.ListIterator;

import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class ProductList {
	private List<Product> productList = new ArrayList<>();
	public int nbProducts = 0;
	ControlBoard controlBoardFrame;
	
	private final static String FILE_PRODUCTS_NAME = "Products list.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_PRODUCTS_LIST_HEADER = 	
														"Product name"+CSV_SEPARATOR 
														+ "State"+CSV_SEPARATOR
														+ "Type"+CSV_SEPARATOR
														+ "Min price"+CSV_SEPARATOR
														+ "Max price"+CSV_SEPARATOR
														+ "Min qty"+CSV_SEPARATOR
														+ "Max qty"+CSV_SEPARATOR
														+ "Currency"+CSV_SEPARATOR
														+ "Time Zone"
														+ NEW_LINE_SEPARATOR;
	
	// constructor
	public ProductList(ControlBoard controlBoardFrame) {
		this.controlBoardFrame = controlBoardFrame;
	}
	
	// ***************************************************
	// *** WriteCSVFile
	// ***************************************************	
	public void WriteCSVFile() {
		try {
			FileWriter fileWriter = new FileWriter(FILE_PRODUCTS_NAME);
			fileWriter.append(FILE_PRODUCTS_LIST_HEADER);
			
			for (Product next : productList) {
				String row = 	next.name + CSV_SEPARATOR
								+ next.state + CSV_SEPARATOR		
								+ next.type + CSV_SEPARATOR
								+ next.minPx+ CSV_SEPARATOR
								+ next.maxPx + CSV_SEPARATOR
								+ next.minQty + CSV_SEPARATOR
								+ next.maxQty+ CSV_SEPARATOR
								+ next.currency+ CSV_SEPARATOR
								+ next.timeZone
								+ NEW_LINE_SEPARATOR;
				fileWriter.append(row);
				
			}			
			fileWriter.flush();
			fileWriter.close();
			
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = "the " + FILE_PRODUCTS_NAME + " file cannot be accessed : please check it is well closed.";
			JOptionPane.showMessageDialog(controlBoardFrame, msg);
		}		
	}

	// ************************************************************************
	// *** Update the request_response GUI table 
	// ************************************************************************
	public void updateGUITable(GUI_Panel products_GUI_Panel) {
		int productRow = 0;
		boolean alreadySortedOnce = false;
		int columnNb = 0;
		JTable table = products_GUI_Panel.table;
		JTextField commentText = products_GUI_Panel.commentText;
		DefaultTableModel dataModel = products_GUI_Panel.dataModel;
		
	if (!products_GUI_Panel.bPaused) {
		commentText.setText("Nb of products = " + this.nbProducts);

	ListIterator<Product> list_Iter = productList.listIterator(); 
	while (list_Iter.hasNext()) {
		Product product_Itr = list_Iter.next();
			
		  // extend the table when reaching the end
		  if  (productRow == table.getRowCount()) dataModel.addRow(new Object[] { productRow });
		  
		  columnNb = 0; // {"#", "Product Name", "State", "Type", 
			//"Min Price", "Max Price", "Min Qty", "Max Qty", "Currency", "Time Zone"};{"#","Reception Time", "Sending Time","Type", "Routing Key",	  "Sequence No", "XML payload"};
		  table.setValueAt(String.format("%02d",productRow+1),productRow,columnNb); columnNb++ ;
		  table.setValueAt(product_Itr.name,productRow,columnNb); columnNb++;
		  table.setValueAt(product_Itr.state,productRow,columnNb); columnNb++ ;
		  table.setValueAt(product_Itr.type,productRow,columnNb); columnNb++ ;
		  table.setValueAt(product_Itr.maxPx,productRow,columnNb); columnNb++ ;
		  table.setValueAt(product_Itr.minPx,productRow,columnNb); columnNb++ ;
		  table.setValueAt(product_Itr.maxQty,productRow,columnNb); columnNb++ ;
		  table.setValueAt(product_Itr.minQty,productRow,columnNb); columnNb++ ;
		  table.setValueAt(product_Itr.currency,productRow,columnNb); columnNb++ ;
		  table.setValueAt(product_Itr.timeZone,productRow,columnNb); columnNb++ ;
		  
		  productRow++;
 	  }
	 
	 // only sort once so that the user can change the sorting 
	 if (alreadySortedOnce == false) {
			table.getRowSorter().toggleSortOrder(1);
			//table.getRowSorter().toggleSortOrder(1);
		 alreadySortedOnce = true;
	 }
	}
	}
	
	// ***************************************************
	// *** Add a product to the list 
	// ***************************************************
	public void AddProduct(Product productToAdd) {
		
		productList.add(productToAdd);
		this.nbProducts ++;
	}
	
	// ***************************************************
	// *** Update product  
	// ***************************************************
	public void UpdateProduct(Product productToAdd) {
		boolean bProduct_Found = false;
		ListIterator<Product> list_Iter = productList.listIterator(); 
		while (list_Iter.hasNext()) {
			Product product_Itr = list_Iter.next();
			if (product_Itr.name.contentEquals(productToAdd.name)) {
				bProduct_Found = true;
			}
		}
		
		if (!bProduct_Found) {
			productList.add(productToAdd);
			this.nbProducts ++;			
		}

	}

	// ***************************************************
	// *** Build list of products for XML requests 
	// ***************************************************
	public String BuildXMLActiveProductsList() {
		String productListString = "";
		ListIterator<Product> list_Iter = productList.listIterator(); 
		while (list_Iter.hasNext()) {
			Product product_Itr = list_Iter.next();
			if (product_Itr.state.contentEquals("ACTI")) productListString = productListString + "<prodName>" + product_Itr.name + "</prodName>";
		}
		
	return productListString;
	}
}
