import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.ListIterator;
import java.util.concurrent.CopyOnWriteArrayList;

import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class RequestResponseList {
	private CopyOnWriteArrayList<RequestResponse> request_responseList = new CopyOnWriteArrayList<RequestResponse>();
	private SimpleDateFormat sdf = new SimpleDateFormat("yyy/MM/dd HH:mm:ss.sss") ;
	private int nbRequestResponses = 0;
	private final static String REQUESTS_RESPONSES_TYPE_PREFIX = "com.deutscheboerse.m7.trading.api.v6.";
	private boolean alreadySortedOnce = false;
	private SimpleDateFormat sdfTime = new SimpleDateFormat("HH:mm:ss.SSS") ;
	private ControlBoard controlBoardFrame;
	private Log appLog;
	
	private final static String FILE_REQUESTS_RESPONSES_LIST_NAME = "Requests Responses.csv";
	private final static String NEW_LINE_SEPARATOR ="\n";
	private final static String CSV_SEPARATOR =";";
	private final static String FILE_REQUESTS_RESPONSES_LIST_HEADER = 
														"Client Reception Time"+CSV_SEPARATOR
														+ "AMQP Sending Time"+CSV_SEPARATOR
														+ "Request/Response" + CSV_SEPARATOR
														+ "Type"+CSV_SEPARATOR
														+ "State"+CSV_SEPARATOR
														+ "RTT (seconds)" + CSV_SEPARATOR
														+ "Routing Key"+CSV_SEPARATOR
														+ "Correlation Id"+CSV_SEPARATOR
														+ "XML Payload"
														+ NEW_LINE_SEPARATOR;

	
	public RequestResponseList(ControlBoard controlBoardFrame, Log appLog) {
		this.controlBoardFrame = controlBoardFrame;
		this.appLog = appLog;
	}
	// ***************************************************
	// *** WriteOrderBooksListCSVFile
	// ***************************************************	
	public void WriteCSVFile() throws Exception {
		try {
	         File f = new File(FILE_REQUESTS_RESPONSES_LIST_NAME);
	         if(!f.exists()) {
	            f.createNewFile();
	         }
	         
			FileWriter fileWriter = new FileWriter(FILE_REQUESTS_RESPONSES_LIST_NAME);
			
			if(f.length() == 0) fileWriter.append(FILE_REQUESTS_RESPONSES_LIST_HEADER);
				
			ListIterator<RequestResponse> list_Iter = request_responseList.listIterator(); 
			while (list_Iter.hasNext()) {
				RequestResponse request_Response_Itr = list_Iter.next();
				
				String row = 	request_Response_Itr.clientSendingORReceptionTime + CSV_SEPARATOR
								+ request_Response_Itr.AMQPSendingTime + CSV_SEPARATOR
								+ request_Response_Itr.request_OR_response + CSV_SEPARATOR
								+ request_Response_Itr.type + CSV_SEPARATOR
								+ request_Response_Itr.state + CSV_SEPARATOR
								+ request_Response_Itr.clientRTT + CSV_SEPARATOR
								+ request_Response_Itr.routingKey + CSV_SEPARATOR
								+ request_Response_Itr.correlationId + CSV_SEPARATOR
								+ request_Response_Itr.XMLPayload
								+ NEW_LINE_SEPARATOR;
				fileWriter.append(row);
				}	
			
			fileWriter.flush();
			fileWriter.close();				
		
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = "the " + FILE_REQUESTS_RESPONSES_LIST_NAME + " file cannot be accessed : please check it is well closed."; 
			JOptionPane.showMessageDialog(controlBoardFrame, msg);
			
		}
	}
	
	// ***************************************************
	// *** Add a request_response to the list 
	// ***************************************************
	public void AddRequestResponse (String request_OR_response,	String contentEncoding, String clientSendingORReceptionTime, String AMQPSendingTime, String type, String routingKey, 
									String correlationId,
									String XMLPayload,
									boolean bresponseReceptionMandatoryToStart) throws Exception {
		String truncatedType = "";
		RequestResponse request_response = new RequestResponse();
		request_response.requestORResponseNb = String.format("%05d",this.request_responseList.size()+1);

		if (request_OR_response.contains("REQUEST")) {
			String clientSendingTime = sdfTime.format(new Date());
			request_response.clientSendingORReceptionTime = clientSendingTime; 
			//request_response.responseReceivedTime=;
			request_response.state = Globals.REQ_STATE_WAITING;
			request_response.clientRTT = "...";
			request_response.clientSendingTime_Long = System.nanoTime();
			request_response.bresponseReceptionMandatoryToStart = bresponseReceptionMandatoryToStart;
			request_response.responseReceivedTime="-";
			request_response.contentEncoding = contentEncoding;
		}
		else  { // when adding a RESPONSE
			request_response.responseReceivedTime=sdfTime.format(new Date());
			request_response.clientSendingORReceptionTime = clientSendingORReceptionTime;
			request_response.clientRTT = "-";
			request_response.contentEncoding = contentEncoding;
		}

		request_response.AMQPSendingTime = AMQPSendingTime;
		request_response.request_OR_response = request_OR_response;
		
		if (type.contains(REQUESTS_RESPONSES_TYPE_PREFIX)) {
			truncatedType = type.substring(type.indexOf(REQUESTS_RESPONSES_TYPE_PREFIX) + REQUESTS_RESPONSES_TYPE_PREFIX.length());			
		}
		else {
			truncatedType = type;
		}

		request_response.correlationId = correlationId;
		
		request_response.type = truncatedType;
		request_response.routingKey = routingKey;
		request_response.XMLPayload = XMLPayload;
		
		request_responseList.add(request_response);
		this.nbRequestResponses ++;
	}
	

	// ************************************************************************
	// *** Update the request_response GUI table 
	// ************************************************************************
	public void updateGUITable(GUI_Panel request_response_GUI_Panel) throws Exception {
		int request_responseRow = 0;
		int columnNb = 0;
		
		boolean alreadySortedOnce = false;
		JTable table = request_response_GUI_Panel.table;
		JTextField commentText = request_response_GUI_Panel.commentText;
		DefaultTableModel dataModel = request_response_GUI_Panel.dataModel;
		
	if (!request_response_GUI_Panel.bPaused) {
		commentText.setText("Nb of request_responses = " + this.nbRequestResponses);

	ListIterator<RequestResponse> list_Iter = request_responseList.listIterator(); 
	while (list_Iter.hasNext()) {
		RequestResponse request_Response_Itr = list_Iter.next();
			
		  // extend the table when reaching the end
		  if  (request_responseRow == table.getRowCount()) dataModel.addRow(new Object[] { request_responseRow });
		  
		  columnNb = 0; // {"#","Reception Time", "Sending Time","Type", "Routing Key",	  "Sequence No", "XML payload"};
		  //table.setValueAt( request_responseRow+1,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.requestORResponseNb,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.clientSendingORReceptionTime,request_responseRow,columnNb); columnNb++;
		  table.setValueAt(request_Response_Itr.AMQPSendingTime,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.responseReceivedTime,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.request_OR_response,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.type,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.state,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.contentEncoding,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.clientRTT,request_responseRow,columnNb); columnNb++ ;
		  if (request_Response_Itr.bresponseReceptionMandatoryToStart) {
			  table.setValueAt("true",request_responseRow,columnNb);   
		  }
		  else table.setValueAt("-",request_responseRow,columnNb); 
		  columnNb++ ;		  
		  table.setValueAt(request_Response_Itr.routingKey,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.correlationId,request_responseRow,columnNb); columnNb++ ;
		  table.setValueAt(request_Response_Itr.XMLPayload,request_responseRow,columnNb); columnNb++ ;
		  
		  request_responseRow++;
 	  }
	 
	 // only sort once so that the user can change the sorting 
	 if (alreadySortedOnce == false) {
		 try {
			table.getRowSorter().toggleSortOrder(1);
			table.getRowSorter().toggleSortOrder(1);
		 alreadySortedOnce = true;
		 }
		 catch (Exception e) {
			 e.printStackTrace();
		 }
	 }
	}
	}
	
	// ***************************************************
	// *** updateRequestWithResponseTime
	// ***************************************************
	public void updateRequestWithResponseTime(String correlationId, String type, String state) {
		Long clientReceptionTm_Long = System.nanoTime();

		// update the request to indicate whether or not a response with the same correlation ID has been received, with the indicated state (see Globals.REQ_STATE_XXX)
		ListIterator<RequestResponse> list_Iter = request_responseList.listIterator(); 
		while (list_Iter.hasNext()) {
			RequestResponse request_Response_Itr = list_Iter.next();
			  if (request_Response_Itr.request_OR_response.contentEquals("REQUEST") && request_Response_Itr.correlationId.contentEquals(correlationId)) {
				  request_Response_Itr.responseReceivedTime = sdfTime.format(new Date());
				  request_Response_Itr.state = state;
				  request_Response_Itr.clientRTT = String.format("%.3f",(clientReceptionTm_Long - request_Response_Itr.clientSendingTime_Long) / 1e9);
				  if (type.contains("ErrResp")) {
					  request_Response_Itr.state = Globals.REQ_STATE_ERROR_RESP;
				  }
				  break;
			  }
		  }
		// update the response
		ListIterator<RequestResponse> list_Iter_Response = request_responseList.listIterator(); 
		while (list_Iter_Response.hasNext()) {
			RequestResponse request_Response_Itr = list_Iter_Response.next();
			  if (request_Response_Itr.request_OR_response.contentEquals("RESPONSE") && request_Response_Itr.correlationId.contentEquals(correlationId)) {
				  request_Response_Itr.state = state;
				  break;
			  }
		  }
		
	}
	
	// ************************************************************************
	// *** CheckAllMandatoryResponsesReceived
	// ************************************************************************
	public boolean StillWaitingForMandatoryResponses() {
		boolean stillWaitingForMandatory = false; 

		ListIterator<RequestResponse> list_Iter = request_responseList.listIterator(); 
		while (list_Iter.hasNext()) {
			RequestResponse request_Response_Itr = list_Iter.next();
			  if (request_Response_Itr.request_OR_response.contentEquals("REQUEST") && request_Response_Itr.bresponseReceptionMandatoryToStart && !request_Response_Itr.state.contentEquals(Globals.REQ_STATE_PROCESSED)) {
				  String msg = request_Response_Itr.request_OR_response + " " + request_Response_Itr.type + " : resp  mandatory = "  + request_Response_Itr.bresponseReceptionMandatoryToStart  + " state = "+ request_Response_Itr.state;
				  appLog.logger.info(msg);
				  Globals.guiWindowDebug.AddText(msg);
				  stillWaitingForMandatory = true;
				  break;
			  }
		  }
		Globals.bBroadcastProcessingCanStart = !stillWaitingForMandatory;
		return stillWaitingForMandatory;
	}
}
