import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.ListIterator;
import java.util.Random;

import javax.swing.JOptionPane;

public class ThreadFillOrderBooks implements Runnable {
	SampleCode sampleCode;
	ControlBoard controlBoardFrame;
	String fillEmptyOrderBooks;
	OrderBooksList orderBooksList;
	Log appLog;
	String userDefaultBalancingGroup;
	WindowOGStartStop windowOrderGenerationControl;
	boolean adminMode;
	private SimpleDateFormat sdfTime = new SimpleDateFormat("HH:mm:ss.SSS");
	int orderNb;
	
	// constructor
	public ThreadFillOrderBooks(SampleCode sampleCode, ControlBoard controlBoardFrame, Log appLog, WindowOGStartStop windowOrderGenerationControl, String fillEmptyOrderBooks, OrderBooksList orderBooksList, String userDefaultBalancingGroup) {
		this.sampleCode = sampleCode;
		this.controlBoardFrame = controlBoardFrame;
		this.fillEmptyOrderBooks = fillEmptyOrderBooks;
		this.orderBooksList = orderBooksList;
		this.appLog = appLog;
		this.userDefaultBalancingGroup = userDefaultBalancingGroup;
		this.windowOrderGenerationControl = windowOrderGenerationControl;
		this.adminMode = sampleCode.adminMode;
		this.orderNb = sampleCode.orderNb;
	}
	
	@Override
	// thread start
	public synchronized void run() {
		try {
			FillInEmptyTradableOrderBooks();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String msg = "Catch Error: ThreadFillOrderBooks:  FillInEmptyTradableOrderBooks: " + e.toString();
			appLog.logger.severe(msg);
			JOptionPane.showMessageDialog(controlBoardFrame, msg);
			
		}
	}	
	
	// ************************************************************
	// FillInEmptyTradableOrderBooks
	// Submit a buy and sell order for each tradable empty order book
	// ************************************************************
	public void FillInEmptyTradableOrderBooks() throws Exception {
	
		appLog.logger.info("fillEmptyOrderBooks = " + fillEmptyOrderBooks + ", nb of order books = "
				+ orderBooksList.getNbOrderBooks());
		int orderNbFillOBK = 0;
		String BGFillObk = userDefaultBalancingGroup;
		int nbObkScan = 0;
		
		int nbEmptyObk = orderBooksList.nbTradableOrderBooks;
		int nbEmptyTradableObk = orderBooksList.nbEmptyTradableOrderBooks;
		float ratioEmptyObk = ((float) nbEmptyTradableObk / (float) nbEmptyObk) * 100;

		String msg = "Initial empty Order books situation : " +  nbEmptyTradableObk + " / " + nbEmptyObk + " (" + String.format("%.0f%%", ratioEmptyObk) + ")";

		appLog.logger.info(msg);
		controlBoardFrame.AddText(msg, controlBoardFrame.textAreaConnection);
		//JOptionPane.showMessageDialog(controlBoardFrame, msg);
		
		// infinite loop when logged in as a trader if indicated in config file to fill
		// in empty order books
		// JOptionPane.showMessageDialog(controlBoardFrame, "fillEmptyOrderBooks:
		// windowOrderGenerationControl.orderGeneration_ON = "+
		// windowOrderGenerationControl.orderGeneration_ON);
		// && windowOrderGenerationControl.orderGeneration_ON
		
		if (adminMode == false && fillEmptyOrderBooks.contentEquals("true")) {
			int initialNbEmptyObk = orderBooksList.getNbEmptyTradableOrderBooks();			
		}
		while (adminMode == false && fillEmptyOrderBooks.contentEquals("true")) {

			// OrdersList ordersList = new OrdersList();
			nbObkScan++;
			appLog.logger.info("nbObkScan = " + nbObkScan);
						
			// wait until there is one tradable obk without any active order:

			// go through the obk list and fill in all tradable obk without any active
			// order: add a buy and sell order
			int obkNbInList = 0;
			int currentNbEmptyObk = 0;
			
			ListIterator<OrderBook> list_Iter = orderBooksList.orderBooksList.listIterator(); 
			while (list_Iter.hasNext() && sampleCode.amqpConnection.ConnectionIsOpen()) {
				OrderBook orderBookItr = list_Iter.next();
				
				obkNbInList++;
				currentNbEmptyObk = orderBooksList.getNbEmptyTradableOrderBooks();
				appLog.logger.fine("Going through the obk list : obk #"+obkNbInList);
				
				// efficiency: do not submit twice in a row orders on the same contract (send to one area: if there is some capacity then connected areas will receive the order as a cross border order for that contract)
				String previousContractId ="" ;
				
				windowOrderGenerationControl.setStrategy(0); // set to systematic order sending
				
				// only process tradable order books without any active order
				if (!"NOT_FOUND".contains(orderBookItr.getContractID_Name()) && orderBookItr.getContractID_Area_Is_Tradable()
						&& orderBookItr.getNbActiveOrders() == 0  && windowOrderGenerationControl.orderGeneration_ON
						&& !"10YLV-1001A00074".contentEquals(orderBookItr.getDlvryAreaId())) { // temporary config issue in ASIM
					
					// && (!orderBookItr.getContractId().contentEquals(previousContractId))
					previousContractId = orderBookItr.getContractId();
					// orderNbFillOBK < 7 &&
					// appLog.logger.severe("Empty obk found #" + obkNbInList+ ": " +
					// orderBookItr.getContractId() + " " + orderBookItr.getContractID_Name() + " " +
					// orderBookItr.getDlvryAreaId() + " initial nb of empty obk = " + initialNbEmptyObk + "
					// current nb ="+ currentNbEmptyObk);

					long previousOrderEntryTime = System.nanoTime();

					// buy order
					orderNbFillOBK++;
					Order buyOrder = new Order();
					// buyOrder.correlationId = start_time+"_FILL#"+orderNbFillOBK;
					buyOrder.correlationId = "LOAD_BUY__FILL_OBK#" + sampleCode.orderNb + "_" + sdfTime.format(new Date());
					buyOrder.clientOrderId = buyOrder.correlationId;
					buyOrder.side = "BUY";
					buyOrder.state = "ACTI";
					buyOrder.BalancingGroup = BGFillObk;
					buyOrder.product = orderBookItr.getContractID_Product();
					buyOrder.contractId = orderBookItr.getContractId();
					buyOrder.contractName = orderBookItr.getContractID_Name();
					buyOrder.deliveryAreaId = orderBookItr.getDlvryAreaId();
					buyOrder.executionRestriction = "NON";
					// type: O for limit Orders, B for User Defined Blocks
					if ("false".contentEquals(orderBookItr.getContractID_Product_Predefined())) {
						buyOrder.executionRestriction = "AON";
						buyOrder.type = "B";
					} else {
						buyOrder.type = "O";
					}
					// random price in a range
					Random randomP = new Random();
					buyOrder.price = randomP.nextInt(5) + 130 + "00";
					buyOrder.quantity = randomP.nextInt(50) + 100 + "000";
					buyOrder.text = "Fill Order Book Order #" + buyOrder.clientOrderId + " contract = "
							+ orderBookItr.getContractID_Name() + " " + buyOrder.deliveryAreaId + " " + BGFillObk
							+ " empty tradable obk = " + orderBooksList.nbEmptyTradableOrderBooks;

					sampleCode.submitOrder(buyOrder);

					// sell order
					orderNbFillOBK++;
					Order sellOrder = new Order();
					sellOrder.correlationId = "LOAD_SELL_FILL_OBK#" + orderNb + "_" + sdfTime.format(new Date());
					sellOrder.clientOrderId = sellOrder.correlationId;
					sellOrder.side = "SELL";
					sellOrder.state = "ACTI";
					sellOrder.BalancingGroup = BGFillObk;
					sellOrder.product = orderBookItr.getContractID_Product();
					sellOrder.contractId = orderBookItr.getContractId();
					sellOrder.contractName = orderBookItr.getContractID_Name();
					sellOrder.deliveryAreaId = orderBookItr.getDlvryAreaId();
					sellOrder.executionRestriction = "NON";
					// type: O for limit Orders, B for User Defined Blocks
					if ("false".contentEquals(orderBookItr.getContractID_Product_Predefined())) {
						sellOrder.executionRestriction = "AON";
						sellOrder.type = "B";
					} else {
						sellOrder.type = "O";
					}
					// random price in a range
					sellOrder.price = Integer.valueOf(buyOrder.price) + 100 + "";
					sellOrder.quantity = randomP.nextInt(50) + 100 + "000";
					sellOrder.text = "Fill Order Book Order #" + sellOrder.clientOrderId + " contract = "
							+ orderBookItr.getContractID_Name() + " " + sellOrder.deliveryAreaId + " " + BGFillObk
							+ " empty obk = " + orderBooksList.nbEmptyTradableOrderBooks;

					sampleCode.submitOrder(sellOrder);
					

					// appLog.logger.severe("Submitting buy and sell orders : contract " +
					// orderBookItr.getContractID_Name() + " area = " + orderBookItr.getDlvryAreaId() + " qty = " +
					// randomQty + " price = " + randomPrice + " ExeRes =" + ExeRes + " Text="+
					// text);

					sampleCode.waitForRemainingTimeBetweenTwoOrderEvents(previousOrderEntryTime);

					if (orderNbFillOBK > 100000) {
						appLog.logger.warning(" nb of orders to fill in empty obk too high: " + orderNbFillOBK + " : EXIT");
						System.exit(0);
					}
				} else {
					// .contentEquals("NOT_FOUND") &&
					msg = "Order book #" + obkNbInList + ":" + orderBookItr.getContractID_Name()
					+ " does not require any order : not empty (" + orderBookItr.getNbActiveOrders()
					+ " orders) or not tradable: orderBookItr.getContractID_Area_Is_Tradable()="
					+ orderBookItr.getContractID_Area_Is_Tradable() + " orderBooksList.getNbEmptyTradableOrderBooks()="
					+ orderBooksList.getNbEmptyTradableOrderBooks();
					//appLog.logger.info(msg);
				}
				// appLog.logger.severe("nbObkScan = "+ nbObkScan + " obkNbInList = " +
				// obkNbInList + " initialNbEmptyObk = " + initialNbEmptyObk + " current
				// orderBooksList.getNbEmptyTradableOrderBooks()="+
				// orderBooksList.getNbEmptyTradableOrderBooks());
			} // loop on all order books in orderBooksList
			appLog.logger.severe("No more empty tradable order books in orderBooksList: obkNbInList = " + obkNbInList);
			appLog.logger.severe("end of obk scan iteration #" + nbObkScan);
		} // infinite loop

	}

}
