import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class ThreadUpdateOrderBooksGUI implements Runnable {

	private OrderBooksList orderBooksList;
	private GUI_Panel orderBooks_GUI_Panel;
	private Log appLog;
	private SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss") ;
	private EmptyOrderBooksList emptyOrderBooksList;
	private DrawEmptyObkFrame emptyObkChart;
	
	// ******************
	// constructor
	// ******************
	public ThreadUpdateOrderBooksGUI(OrderBooksList orderBooksList, DrawEmptyObkFrame emptyObkChart, EmptyOrderBooksList emptyOrderBooksList, GUI_Panel orderBooks_GUI_Panel, Log appLog) {
		this.orderBooksList = orderBooksList;
		this.orderBooks_GUI_Panel = orderBooks_GUI_Panel;
		this.appLog = appLog;
		this.emptyOrderBooksList = emptyOrderBooksList;
		this.emptyObkChart =  emptyObkChart;
	}
	
	// ******************
	// run
	// ******************
	@Override
	public void run() {
		while (true) {
			try {
				updateOrderBookGUI();
			}
		 catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			String sError = "catching error in ThreadUpdateOrderBooksGUI: " + e.toString();
			appLog.logger.severe(sError);
		}
			try {
				Thread.sleep(3000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				String sError = "catching error in ThreadUpdateOrderBooksGUI: " + e.toString();
				appLog.logger.severe(sError);
			}
		}
		
	}
	// *********************
	// update Order Book GUI
	// *********************
	public synchronized void updateOrderBookGUI() throws Exception {
		int orderBookRow = 0;
		int columnNb = 0;
		boolean alreadySortedOnce = false;
   
		JTable orderBooksTable = orderBooks_GUI_Panel.table;
		JTextField commentTextOrderBooks = orderBooks_GUI_Panel.commentText;
		DefaultTableModel dataModel = orderBooks_GUI_Panel.dataModel;
		
	    //int nbTradableObk2 = this.orderBooksList.nbTradableOrderBooks;
	    //int nbEmptyTradableObk2 = this.orderBooksList.nbEmptyTradableOrderBooks;
	    int nbTradableObk2 = this.orderBooksList.countTradableOrderBooks();
	    int nbEmptyTradableObk2 = this.orderBooksList.countTradableEmptyOrderBooks();
	    emptyOrderBooksList.AddEmptyObk(nbEmptyTradableObk2);
	    float ratioEmptyObk2;
	    if (nbTradableObk2>0) ratioEmptyObk2 = ((float)nbEmptyTradableObk2/(float)nbTradableObk2)*100;
	    else ratioEmptyObk2 = 0;
	    
	    String comment = sdf.format(new Date()) +": Nb tradable empty order books = " + nbEmptyTradableObk2 + " / " + nbTradableObk2 + " (" + String.format("%.0f%%",ratioEmptyObk2)+ ")";
	    
		commentTextOrderBooks.setText(comment);

		for (OrderBook next : orderBooksList.orderBooksList) {

			// extend the table when reaching the end 
			if  (orderBookRow == orderBooksTable.getRowCount()-1) dataModel.addRow(new Object[] { orderBookRow });
			
			// only process tradable order books without any active order 
			if (next.getContractID_Area_Is_Tradable()) {
				// only update relevant values from the left to the right
				columnNb = 0;				
				// {"Contract Id", "Contract Name", "Area Id", "Nb of orders","Area phase", "Area State"};
				orderBooksTable.setValueAt(orderBookRow+1,orderBookRow,columnNb); columnNb++ ;
				
				orderBooksTable.setValueAt(next.getContractId(),orderBookRow,columnNb); columnNb++ ;
				orderBooksTable.setValueAt(next.getDlvryAreaId(),orderBookRow,columnNb); columnNb++ ;
				
				orderBooksTable.setValueAt(next.getContractID_Product(),orderBookRow,columnNb); columnNb++ ;
				orderBooksTable.setValueAt(next.getContractID_Name(),orderBookRow,columnNb); columnNb++ ;
				orderBooksTable.setValueAt(next.getDlvryAreaName(),orderBookRow,columnNb); columnNb++ ;

				// //"Best Bid", "Best Ask" 
				String bestBid;
				String bestAsk;
				
				if ("".contentEquals( next.getsBestBid() )) {
					bestBid = "-";
				} 	
				else bestBid = String.format("%.2f",next.getfBestBid());
				
				orderBooksTable.setValueAt(bestBid,orderBookRow,columnNb); columnNb++ ;
				
				if ("".contentEquals( next.getsBestAsk() )) {
					bestAsk = "-";
				} 	
				else bestAsk = String.format("%.2f",next.getfBestAsk());
				
				orderBooksTable.setValueAt(bestAsk,orderBookRow,columnNb); columnNb++ ;
				
				orderBooksTable.setValueAt(String.format("%.2f",next.getLastQty()),orderBookRow,columnNb); columnNb++ ;
				orderBooksTable.setValueAt(String.format("%.2f",next.getLastPx()),orderBookRow,columnNb); columnNb++ ;

				
				orderBooksTable.setValueAt(next.getLastUpdateTime(),orderBookRow,columnNb); columnNb++ ;
				orderBooksTable.setValueAt(next.getLastTradeTime(),orderBookRow,columnNb); columnNb++ ;
				orderBooksTable.setValueAt(String.format("%03d", next.getNbActiveOrders()),orderBookRow,columnNb); columnNb++ ;
				
				orderBooksTable.setValueAt(next.getContractID_Area_Is_Tradable(),orderBookRow,columnNb); columnNb++ ;
				orderBooksTable.setValueAt(next.getContractID_Area_Phase(),orderBookRow,columnNb); columnNb++ ;
				orderBooksTable.setValueAt(next.getContractID_Area_State(),orderBookRow,columnNb); columnNb++ ;
				orderBookRow++;
			}
		}
		
		emptyObkChart.repaint();
		emptyOrderBooksList.WritePlayersListCSVFile();
		
		 if (alreadySortedOnce == false) {
			//	orderBooksTable.getRowSorter().toggleSortOrder(8);
				//orderBooksTable.getRowSorter().toggleSortOrder(8);
			alreadySortedOnce = true;
		 }

		//commentTextOrderBooks.setText(orderBooksList.getNbEmptyTradableOrderBooks()+ " empty Order Book(s)");		
		
	}

}
