import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.ListIterator;

import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.DefaultTableModel;

public class ThreadUpdateOrderBooksList implements Runnable {

	private String contractId;
	private String dlvryAreaId;
	private String revisionNo; 
	private float lastPx, lastQty, bestBid, bestAsk;
	private float totalQty;
	private String lastTradeTime;
	private String lowPx, highPx, sBestBid, sBestAsk;
	private int spread;
	private int depthAsk_NbOrders;
	private int depthBid_NbOrders;
	private int deltaNbOrders; 
	private ContractList contractList;
	private Log appLog;
	private OrderBooksList orderBooksList;
	private DeliveryAreaList deliveryAreaList; 
	private GUI_Panel orderBooks_GUI_Panel;
	private ControlBoard controlBoardFrame;
	private SimpleDateFormat sdf = new SimpleDateFormat("yyy/MM/dd HH:mm:ss.sss") ;
		
	// ******************
	// constructor
	// ******************
	public ThreadUpdateOrderBooksList(ControlBoard controlBoardFrame, GUI_Panel orderBooks_GUI_Panel, OrderBooksList orderBooksList, DeliveryAreaList deliveryAreaList,String contractId, String dlvryAreaId, String revisionNo, 
			float bestBid, float bestAsk, String sBestBid, String sBestAsk, float lastPx, float lastQty, float totalQty, String lastTradeTime, String lowPx, String highPx, 
			int spread,int depthAsk_NbOrders, int depthBid_NbOrders, int deltaNbOrders, 
			ContractList contractList,
			 Log appLog) {
		this.contractId = contractId;
		this.dlvryAreaId = dlvryAreaId;
		this.revisionNo = revisionNo;
		
		this.bestAsk = bestAsk;
		this.sBestAsk = sBestAsk;

		this.bestBid = bestBid;
		this.sBestBid = sBestBid;
		
		this.lastPx = lastPx;
		this.lastQty = lastQty;
		this.totalQty = totalQty;
		this.lastTradeTime = lastTradeTime;
		this.lowPx = lowPx;
		this.highPx = highPx;
		this.spread = spread;
		this.depthAsk_NbOrders = depthAsk_NbOrders;
		this.depthBid_NbOrders = depthBid_NbOrders;
		this.deltaNbOrders = deltaNbOrders;
		this.contractList = contractList;
		this.appLog = appLog;
		this.orderBooksList = orderBooksList;
		this.deliveryAreaList = deliveryAreaList;
		this.orderBooks_GUI_Panel = orderBooks_GUI_Panel;
		this.controlBoardFrame = controlBoardFrame;
	}
	
	// ******************
	// run
	// ******************
	@Override
	public synchronized void run() {
		// System.out.println(" contract Id =" + contractId + " or dlvryAreaId = " + dlvryAreaId + " or contractList = " + contractList + " or orderBooksList = " + orderBooksList + " or orderBooks_GUI_Panel=" + orderBooks_GUI_Panel);
		 if (contractId != null & dlvryAreaId != null && contractList != null && orderBooksList != null && deliveryAreaList != null && orderBooks_GUI_Panel != null) {
			 try {
				UpdateOrderBooksList();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
				e.printStackTrace();
				String msg = "catching ERROR ThreadUpdateOrderBooksList.UpdateOrderBooksList: " + e.toString();
				appLog.logger.severe(msg);
				Globals.guiWindowDebug.AddText(msg);
				//JOptionPane.showMessageDialog(controlBoardFrame, msg);
			}	
			 // obk GUI update replaced by a thread with a periodic update
//			 try {
//				updateOrderBookGUI();
//			} catch (Exception e) {
//				e.printStackTrace();
//				String msg = "catching ERROR ThreadUpdateOrderBooksList.updateOrderBookGUI: " + e.toString();
//				appLog.logger.severe(msg);
//				Globals.guiWindowDebug.AddText(msg);
//				//JOptionPane.showMessageDialog(controlBoardFrame, msg);
//			}
		 }
		else System.out.println(" contract Id =" + contractId + " or dlvryAreaId = " + dlvryAreaId + " or contractList = " + contractList + " or orderBooksList = " + orderBooksList + " or orderBooks_GUI_Panel=" + orderBooks_GUI_Panel + "  is null");
	}
	// *****************************
	// update Order Book List data
	// *****************************
	public synchronized void UpdateOrderBooksList() throws Exception {
			
		boolean bExecute = true;
			
		if (bExecute) {
			boolean Obk_Found = false;
			
			ListIterator<OrderBook> list_Iter = orderBooksList.orderBooksList.listIterator(); 
			while (list_Iter.hasNext()) {
				OrderBook orderBookItr = list_Iter.next();
				
				if ( contractId.contentEquals(orderBookItr.getContractId()) && dlvryAreaId.contentEquals(orderBookItr.getDlvryAreaId())) {
					// BG already listed : just update values
					Obk_Found = true;
		
					orderBookItr.setRevisionNo(revisionNo);
					
					orderBookItr.setfBestBid(bestBid);
					orderBookItr.setfBestAsk(bestAsk);	

					orderBookItr.setsBestBid(sBestBid);
					orderBookItr.setsBestAsk(sBestAsk);
					
					orderBookItr.setHighPx(highPx);
					orderBookItr.setLastPx(lastPx);
					orderBookItr.setLastQty(lastQty);
					orderBookItr.setTotalQty(totalQty);
					orderBookItr.setLastTradeTime(lastTradeTime);
					orderBookItr.setLowPx(lowPx);
					orderBookItr.setHighPx(highPx);

					orderBookItr.setSpread(spread);
					orderBookItr.setDepthAsk_NbOrders(depthAsk_NbOrders);
					orderBookItr.setDepthBid_NbOrders(depthBid_NbOrders);
					
					//nb of active orders: take into account orders exiting the obk (qty =0), because hibernated, deleted or fully traded (or out of order book depth for XBID obks)
					int previousNbActiveOrders = orderBookItr.getNbActiveOrders() ;
					int newNbActiveOrders = orderBookItr.getNbActiveOrders() + deltaNbOrders;
					if (newNbActiveOrders<0) newNbActiveOrders = 0; // potential bug protection
					orderBookItr.setNbActiveOrders(newNbActiveOrders);
					
					orderBookItr.setLastUpdateTime(sdf.format(new Date()));
					
					boolean bPreviousObkTradableState = orderBookItr.getContractID_Area_Is_Tradable();
					
					// find contract related info
					OrderBook  orderBookTemp = new OrderBook(); // using an order book just to pass contract info to an easy object
					orderBookTemp = contractList.FindContractIdInfo (contractId, dlvryAreaId, null);
					
					// orderBookItr.setContractID_Product(orderBookTemp.getContractID_Product()); // product does not change
					orderBookItr.setContractID_Name(orderBookTemp.getContractID_Name());
					orderBookItr.setContractID_Area_Phase(orderBookTemp.getContractID_Area_Phase());
					orderBookItr.setContractID_Area_State(orderBookTemp.getContractID_Area_State());
					orderBookItr.setContractID_Area_Is_Tradable(orderBookTemp.getContractID_Area_Is_Tradable());
					boolean bNewObkTradableState = orderBookTemp.getContractID_Area_Is_Tradable();

					String msg ="order book update: "+ orderBookItr.getContractID_Name()
					+ " bPreviousObkTradableState = " + bPreviousObkTradableState
					+ " bNewObkTradableState = "+ bNewObkTradableState
					+ "  nbTradableOrderBooks = " + orderBooksList.nbTradableOrderBooks
					+ "  nbEmptyTradableOrderBooks = " + orderBooksList.nbEmptyTradableOrderBooks;
					
					appLog.logger.info(msg);
					System.out.println(msg);
					Globals.guiWindowDebug.AddText(msg);
					
//					if (bPreviousObkTradableState != bNewObkTradableState) {
//						if (bNewObkTradableState)  { // the order book has become tradable
//							this.orderBooksList.nbTradableOrderBooks ++;
//							appLog.logger.info("order book update: "+ orderBookItr.getContractID_Name() + " becomes tradable: nbTradableOrderBooks ++ = " + orderBooksList.nbTradableOrderBooks);
//						}
//						else { // the order book is not tradable anymore
//							this.orderBooksList.nbTradableOrderBooks --;
//							appLog.logger.info("order book update: "+ orderBookItr.getContractID_Name() + " NOT tradable anymore: nbTradableOrderBooks -- = " + orderBooksList.nbTradableOrderBooks);
//						}
//					}
//					// if the obk is tradable and nb active orders decreased to zero
//					if ( bNewObkTradableState && (previousNbActiveOrders != newNbActiveOrders)) { 
//						if (previousNbActiveOrders > 0 && newNbActiveOrders == 0) {
//							this.orderBooksList.nbEmptyTradableOrderBooks ++;
//							appLog.logger.info("order book update: "+ orderBookItr.getContractID_Name() + " newNbActiveOrders down to zero = " + newNbActiveOrders + " nbEmptyTradableOrderBooks ++ ="+ orderBooksList.nbEmptyTradableOrderBooks);
//						}
//					}
//					// if the obk is tradable and nb active orders decreased to zero
//					if ( bNewObkTradableState && (previousNbActiveOrders != newNbActiveOrders)) { 
//							if (previousNbActiveOrders ==0  && newNbActiveOrders > 0) {
//								this.orderBooksList.nbEmptyTradableOrderBooks --;
//								appLog.logger.info("order book update: "+ orderBookItr.getContractID_Name() + " newNbActiveOrders increases from zero = " + newNbActiveOrders + " nbEmptyTradableOrderBooks -- ="+ orderBooksList.nbEmptyTradableOrderBooks);
//							}
//					}
				
					break;
				}
			}
			// if Obk could not be found in the list : add it
			if (Obk_Found == false) {
			
				this.AddOrderBook(contractId, dlvryAreaId, revisionNo, 
						bestBid, bestAsk,  sBestBid, sBestAsk, lastPx, lastQty, totalQty, lastTradeTime, lowPx, highPx, 
						spread,depthAsk_NbOrders, depthBid_NbOrders, deltaNbOrders, contractList,appLog);
			}

		}
		}
	// ***************************************************
	// *** Add an order book to the list 
	// ***************************************************
	public synchronized int AddOrderBook(String contractId, String dlvryAreaId, String revisionNo, 
			float bestBid, float bestAsk, String sBestBid, String sBestAsk, float lastPx, float lastQty, float totalQty, String lastTradeTime, String lowPx, String highPx, 
							int spread,int depthAsk_NbOrders, int depthBid_NbOrders, int nbOrders, ContractList contractList,
							 Log appLog) {
		OrderBook  orderBook = new OrderBook();
		
		orderBook.setContractId(contractId);
		orderBook.setDlvryAreaId(dlvryAreaId);
		orderBook.setRevisionNo(revisionNo);
		
		orderBook.setfBestBid(bestBid);
		orderBook.setfBestAsk(bestAsk);
		orderBook.setsBestBid(sBestBid);
		orderBook.setsBestAsk(sBestAsk);
		
		orderBook.setHighPx(highPx);
		orderBook.setLastPx(lastPx);
		orderBook.setLastQty(lastQty);
		orderBook.setTotalQty(totalQty);
		orderBook.setLastTradeTime(lastTradeTime);
		orderBook.setLowPx(lowPx);
		orderBook.setHighPx(highPx);
		
		orderBook.setSpread(spread);
		orderBook.setDepthAsk_NbOrders(depthAsk_NbOrders);
		orderBook.setDepthBid_NbOrders(depthBid_NbOrders);
		orderBook.setNbOrders(nbOrders); 
	
		orderBook.setLastUpdateTime(sdf.format(new Date()));
		
		// find contract related info
	/*	Contract contractInfo = new Contract();
		contractInfo = contractList.FindContractIdInfo_Contract(contractId, Globals.guiWindowDebug);
		orderBook.setContractID_Product(contractInfo.getProd());
		orderBook.setContractID_Product_Predefined(contractInfo.getPredefined()());
		orderBook.setContractID_Product_DeliveryUnits(contractInfo.getDelUnits());
		orderBook.setContractID_Name(contractInfo.getContractID_Area_Phase());
		orderBook.setContractID_Area_Phase(contractInfo.get);
		orderBook.setContractID_Area_State(contractInfo.getContractID_Area_State());
		orderBook.setContractID_Area_Is_Tradable(contractInfo.getContractID_Area_Is_Tradable());	*/	
		
		OrderBook  orderBookTemp = new OrderBook();
		orderBookTemp = contractList.FindContractIdInfo (contractId, dlvryAreaId, null);
		
		orderBook.setContractID_Product(orderBookTemp.getContractID_Product());
		orderBook.setContractID_Product_Predefined(orderBookTemp.getContractID_Product_Predefined());
		orderBook.setContractID_Product_DeliveryUnits(orderBookTemp.getContractID_Product_DeliveryUnits());
		orderBook.setContractID_Name(orderBookTemp.getContractID_Name());
		orderBook.setContractID_Area_Phase(orderBookTemp.getContractID_Area_Phase());
		orderBook.setContractID_Area_State(orderBookTemp.getContractID_Area_State());
		orderBook.setContractID_Area_Is_Tradable(orderBookTemp.getContractID_Area_Is_Tradable());
		
		// find area info
		DeliveryArea areaDetails = new DeliveryArea();
		areaDetails = deliveryAreaList.GetAreaDetailsFromId(dlvryAreaId);
		orderBook.setDlvryAreaName(areaDetails.dlvryAreaName);
		orderBook.setDlvryAreaLongName(areaDetails.dlvryAreaLongName);

		// update stats at order book list level
		if (orderBook.getContractID_Area_Is_Tradable()) {
			orderBooksList.nbTradableOrderBooks ++;
			if (nbOrders == 0) orderBooksList.nbEmptyTradableOrderBooks ++;			
		}
		
		orderBooksList.orderBooksList.add(orderBook);
		orderBooksList.setNbOrderBooks(orderBooksList.getNbOrderBooks() + 1);
		
		return orderBooksList.getNbOrderBooks();
	}
//	// *********************
//	// update Order Book GUI
//	// *********************
//	public synchronized void updateOrderBookGUI()  throws Exception{
//		int orderBookRow = 0;
//		int columnNb = 0;
//		boolean alreadySortedOnce = false;
//		
//		JTable orderBooksTable = orderBooks_GUI_Panel.table;
//		JTextField commentTextOrderBooks = orderBooks_GUI_Panel.commentText;
//		DefaultTableModel dataModel = orderBooks_GUI_Panel.dataModel;
//		
//	    int nbTradableObk2 = this.countTradableOrderBooks();
//	    int nbEmptyTradableObk2 = this.countTradableEmptyOrderBooks();
//	    
//	    orderBooksList.nbTradableOrderBooks = nbTradableObk2;
//	    orderBooksList.nbEmptyTradableOrderBooks = nbEmptyTradableObk2;
//	    float ratioEmptyObk2;
//	    if (nbTradableObk2>0) ratioEmptyObk2 = ((float)nbEmptyTradableObk2/(float)nbTradableObk2)*100;
//	    else ratioEmptyObk2 = 0;
//	    
//	    String comment = "Nb tradable empty order books = " + nbEmptyTradableObk2 + " / " + nbTradableObk2 + " (" + String.format("%.0f%%",ratioEmptyObk2)+ ")";
//	    
//		commentTextOrderBooks.setText(comment);
//
//		ListIterator<OrderBook> list_Iter = orderBooksList.orderBooksList.listIterator(); 
//		while (list_Iter.hasNext()) {
//			OrderBook orderBookTemp = list_Iter.next();
//			// extend the table when reaching the end 
//			if  (orderBookRow == orderBooksTable.getRowCount()-1) dataModel.addRow(new Object[] { orderBookRow });
//			
//			
//			//{"#", "Contract Id",  "Area Id", "Product",  "Contract Name","Area Name",
//				
//			// only process tradable order books without any active order 
//			if (orderBookTemp.getContractID_Area_Is_Tradable()) {
//				// only update relevant values from the left to the right
//				columnNb = 0;				
//				orderBooksTable.setValueAt(orderBookRow+1,orderBookRow,columnNb); columnNb++ ;
//				
//				orderBooksTable.setValueAt(orderBookTemp.getContractId(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookTemp.getDlvryAreaId(),orderBookRow,columnNb); columnNb++ ;
//				
//				orderBooksTable.setValueAt(orderBookTemp.getContractID_Product(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookTemp.getContractID_Name(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookTemp.getDlvryAreaName(),orderBookRow,columnNb); columnNb++ ;
//				
//				// //"Best Bid", "Best Ask" 
//				String bestBid;
//				String bestAsk;
//				
//				if ("".contentEquals( orderBookTemp.getsBestBid() )) {
//					bestBid = "-";
//				} 	
//				else bestBid = String.format("%.2f",orderBookTemp.getfBestBid());
//				
//				orderBooksTable.setValueAt(bestBid,orderBookRow,columnNb); columnNb++ ;
//				
//				if ("".contentEquals( orderBookTemp.getsBestAsk() )) {
//					bestAsk = "-";
//				} 	
//				else bestAsk = String.format("%.2f",orderBookTemp.getfBestAsk());
//				
//				orderBooksTable.setValueAt(bestAsk,orderBookRow,columnNb); columnNb++ ;
//		
//				//"Last qty","Last price",  "last update", "last trade", "Nb of orders",
//				
//				orderBooksTable.setValueAt(String.format("%.2f",orderBookTemp.getLastQty()),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(String.format("%.2f",orderBookTemp.getLastPx()),orderBookRow,columnNb); columnNb++ ;
//
//				orderBooksTable.setValueAt(orderBookTemp.getLastUpdateTime(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookTemp.getLastTradeTime(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(String.format("%03d", orderBookTemp.getNbActiveOrders()),orderBookRow,columnNb); columnNb++ ;
//				
//				// "Tradable","Area phase", "Area State"};
//				orderBooksTable.setValueAt(orderBookTemp.getContractID_Area_Is_Tradable(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookTemp.getContractID_Area_Phase(),orderBookRow,columnNb); columnNb++ ;
//				orderBooksTable.setValueAt(orderBookTemp.getContractID_Area_State(),orderBookRow,columnNb); columnNb++ ;
//				orderBookRow++;
//			}
//		}
//		
//		 if (alreadySortedOnce == false) {
//			//	orderBooksTable.getRowSorter().toggleSortOrder(8);
//				//orderBooksTable.getRowSorter().toggleSortOrder(8);
//			alreadySortedOnce = true;
//		 }
//
//		//commentTextOrderBooks.setText(orderBooksList.getNbEmptyTradableOrderBooks()+ " empty Order Book(s)");		
//		
//	}

	
	// ************************************************************************
	// *** Count how many tradable order books do not have any order
	// ************************************************************************
	public int countTradableEmptyOrderBooks() {
		int nbEmptyTradableOrderBooks = 0;
		
		ListIterator<OrderBook> list_Iter = orderBooksList.orderBooksList.listIterator(); 
		while (list_Iter.hasNext()) {
			OrderBook orderBookItr = list_Iter.next();
			
			if (orderBookItr.getContractID_Area_Is_Tradable() && orderBookItr.getNbActiveOrders() == 0) {
				nbEmptyTradableOrderBooks ++;
			}
		}
	//	orderBooksList.nbEmptyTradableOrderBooks = nbEmptyTradableOrderBooks;
		return nbEmptyTradableOrderBooks;		
	}
	// ************************************************************************
	// *** Count how many tradable order books 
	// ************************************************************************
	public int countTradableOrderBooks() {
		int nbTrdableOrderBooks = 0;
		ListIterator<OrderBook> list_Iter = orderBooksList.orderBooksList.listIterator(); 
		while (list_Iter.hasNext()) {
			OrderBook orderBookItr = list_Iter.next();
			if (orderBookItr.getContractID_Area_Is_Tradable() ) {
				nbTrdableOrderBooks ++;
			}
		}
		//orderBooksList.nbTradableOrderBooks = nbTrdableOrderBooks;
		return nbTrdableOrderBooks;		
	}
}
